import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib, math
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	inner_diameter = params['INNER_DIAMETER']
	length = params['LENGTH']
	outer_diameter = params['OUTER_DIAMETER']
	chamfer = params['CHAMFER']
	angle_chamfer = params['ANGLE_CHAMFER']
	invert_element = params['INVERT_ELEMENT']
	clearence_diameter = params['CLEARENCE_DIAMETER']
	offset = params['OFFSET']
	plate_thick = params['PLATE_THICK']

	Uni_list = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	if invert_element:
		VISIGeoLib.ProjectPointOntoVector(start_pos, direction, plate_thick-length, start_pos)
	
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, offset, start_pos)
	
	
	cyl1 = VISISolidFactory.CreateCylinder(outer_diameter/2, length, start_pos, x_axis, direction)
	cyl2 = VISISolidFactory.CreateCylinder(inner_diameter/2, length, start_pos, x_axis, direction)
	
	Uni_list.append(cyl1)
	Uni_list.append(cyl2)
	
	VISISolidFactory.Subtract(Uni_list, Result_List)
	
	edges=[6,12,30,31]
	distance_2=chamfer*math.tan(angle_chamfer)
	VISISolidFactory.ChamferEdges(Result_List[0], edges, chamfer, distance_2, 0)
	
	## return the tag of created element, 0 if no element
	return Result_List[0]

def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	inner_diameter = params['INNER_DIAMETER']
	length = params['LENGTH']
	outer_diameter = params['OUTER_DIAMETER']
	chamfer = params['CHAMFER']
	angle_chamfer = params['ANGLE_CHAMFER']
	invert_element = params['INVERT_ELEMENT']
	clearence_diameter = params['CLEARENCE_DIAMETER']
	offset = params['OFFSET']
	plate_thick = params['PLATE_THICK']
	
	Uni_list = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	cyl1 = VISISolidFactory.CreateCylinder((outer_diameter+clearence_diameter)/2, plate_thick, start_pos, x_axis, direction)
	cam_list1 = [outer_diameter+clearence_diameter,plate_thick]
	VISI.AttachCamAttributes(cyl1,cam_list1,"BC1_1",elem_tag,params["BUILDER_UNIT"])
	
	## return the tag of created cavity, 0 if no cavity
	return cyl1

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_bc1_1'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	ref_direction = params['REF_DIRECTION']
	Xaxis.Set(ref_direction.Cx(),ref_direction.Cy(),ref_direction.Cz())

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
