import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	width = params['WIDTH']
	length = params['LENGTH']
	height = params['HEIGHT']
	width_holes = params['WIDTH_HOLES']
	distance_hole = params['DISTANCE_HOLE']
	diameter_small = params['DIAMETER_SMALL']
	diameter_big = params['DIAMETER_BIG']
	depth_counterbore = params['DEPTH_COUNTERBORE']
	height_counter = params['MASTER_HEIGHT']
	bottom_flushed = params['BOTTOM_FLUSHED']
	offset_to_body = params['OFFSET_TO_BODY']
	master_ins_depth = params['MASTER_INS_DEPTH']
	
	Uni_list = []
	Uni_list2 = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	y_axis = VISIVector.New()
	y_axis.Set(ref_direction.Cx(), ref_direction.Cy(), ref_direction.Cz())
	x_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, y_axis, x_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	if bottom_flushed:
		VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, height_counter, start_pos)
	else:
		VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, height, start_pos)
	
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, height_counter-master_ins_depth, start_pos)
	
	#Move to position and create main body
	VISIGeoLib.ProjectPointOntoVector(start_pos, y_axis, distance_hole+offset_to_body, start_pos)
	bl1 = VISISolidFactory.CreateBlock(width, length, height, start_pos, x_axis, direction)
	#Create drilling + counterboring on first side
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, height, start_pos)
	VISIGeoLib.ProjectPointOntoVector(start_pos, x_axis, width_holes/2, start_pos)
	cyl1 = VISISolidFactory.CreateCylinder(diameter_small/2, height, start_pos, x_axis, inv_dir)
	cyl2 = VISISolidFactory.CreateCylinder(diameter_big/2, depth_counterbore, start_pos, x_axis, inv_dir)
	#Repeat on the other side
	VISIGeoLib.ProjectPointOntoVector(start_pos, x_axis, -width_holes, start_pos)
	cyl3 = VISISolidFactory.CreateCylinder(diameter_small/2, height, start_pos, x_axis, inv_dir)
	cyl4 = VISISolidFactory.CreateCylinder(diameter_big/2, depth_counterbore, start_pos, x_axis, inv_dir)
	#Subtract cylinders from main body
	Uni_list.append(bl1)
	Uni_list.append(cyl1)
	Uni_list.append(cyl2)
	Uni_list.append(cyl3)
	Uni_list.append(cyl4)
	VISISolidFactory.Subtract(Uni_list, Result_List)

	## return the tag of created element, 0 if no element
	return Result_List[0]

def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	width = params['WIDTH']
	length = params['LENGTH']
	height = params['HEIGHT']
	width_holes = params['WIDTH_HOLES']
	distance_hole = params['DISTANCE_HOLE']
	diameter_small = params['DIAMETER_SMALL']
	diameter_big = params['DIAMETER_BIG']
	depth_counterbore = params['DEPTH_COUNTERBORE']
	height_counter = params['MASTER_HEIGHT']
	bottom_flushed = params['BOTTOM_FLUSHED']
	offset_to_body = params['OFFSET_TO_BODY']
	master_ins_depth = params['MASTER_INS_DEPTH']
	
	Uni_list = []
	Uni_list2 = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	y_axis = VISIVector.New()
	y_axis.Set(ref_direction.Cx(), ref_direction.Cy(), ref_direction.Cz())
	x_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, y_axis, x_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	if bottom_flushed:
		VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, height_counter, start_pos)
	else:
		VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, height, start_pos)
	
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, height_counter-master_ins_depth, start_pos)
	
	#Move to position and create body
	VISIGeoLib.ProjectPointOntoVector(start_pos, y_axis, distance_hole+offset_to_body, start_pos)
	bl1 = VISISolidFactory.CreateBlock(width, length, height, start_pos, x_axis, direction)
	
	
	## return the tag of created cavity, 0 if no cavity
	return bl1

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_bcc1'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	ref_direction = params['REF_DIRECTION']
	Xaxis.Set(ref_direction.Cx(),ref_direction.Cy(),ref_direction.Cz())

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
