import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	diameter_pillar = params['DIAMETER_PILLAR']
	diameter_guide = params['DIAMETER_GUIDE']
	diameter_insertion = params['DIAMETER_INSERTION']
	diameter_flange = params['DIAMETER_FLANGE']
	diameter_top = params['DIAMETER_TOP']
	total_length = params['TOTAL_LENGTH']
	length_guide = params['LENGTH_GUIDE']
	length_out_of_plate = params['LENGTH_OUT_OF_PLATE']
	length_flange = params['LENGTH_FLANGE']
	length_insertion = params['LENGTH_INSERTION']
	radius1 = params['RADIUS1']
	radius2 = params['RADIUS2']
	radius3 = params['RADIUS3']
	chamfer = params['CHAMFER']
	length_cavity = params['LENGTH_CAVITY']
	thickness_plate = params['THICKNESS_PLATE']
	inv_element = params['INV_ELEMENT']

	Uni_list = []
	Uni_list2 = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	direction1 = VISIVector.New()
	direction1.Set(direction.Cx(), direction.Cy(), direction.Cz())
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	#Invert Direction
	if inv_element:
		direction1.Set(inv_dir.Cx(), inv_dir.Cy(), inv_dir.Cz())
		inv_dir.Set(-inv_dir.Cx(), -inv_dir.Cy(), -inv_dir.Cz())
		VISIGeoLib.ProjectPointOntoVector(start_pos, direction1, thickness_plate, start_pos)
	
	#Create bodies
	cyl1 = VISISolidFactory.CreateCylinder(diameter_flange/2, length_flange, start_pos, x_axis, direction1)
	cyl2 = VISISolidFactory.CreateCylinder(diameter_top/2, length_out_of_plate, start_pos, x_axis, direction1)
	cyl3 = VISISolidFactory.CreateCylinder(diameter_guide/2, length_guide-length_insertion, start_pos, x_axis, inv_dir)
	#Move to front and create body
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, length_guide, start_pos)
	cyl4 = VISISolidFactory.CreateCylinder(diameter_insertion/2, length_insertion, start_pos, x_axis, direction1)
	#Unite all bodies
	Uni_list.append(cyl1)
	Uni_list.append(cyl2)
	Uni_list.append(cyl3)
	Uni_list.append(cyl4)
	VISISolidFactory.Unite(Uni_list, Uni_list2)
	#Create body for hole
	cyl5 = VISISolidFactory.CreateCylinder(diameter_pillar/2, total_length, start_pos, x_axis, direction1)
	#Subtract
	Uni_list2.append(cyl5)
	VISISolidFactory.Subtract(Uni_list2, Result_List)
	
	#Modify Edges with Blend/Chamfer
	edge1=[30,34,85]
	VISISolidFactory.BlendEdges(Result_List[0], edge1, radius1, 1)
	edge2=[84]
	VISISolidFactory.BlendEdges(Result_List[0], edge2, radius2, 1)	
	edge3=[24]
	VISISolidFactory.BlendEdges(Result_List[0], edge3, radius3, 1)
	edge4=[6,12]
	VISISolidFactory.ChamferEdges(Result_List[0], edge4, chamfer, chamfer, 1)
	
	
	## return the tag of created element, 0 if no element
	return Result_List [0]

def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	diameter_pillar = params['DIAMETER_PILLAR']
	diameter_guide = params['DIAMETER_GUIDE']
	diameter_insertion = params['DIAMETER_INSERTION']
	diameter_flange = params['DIAMETER_FLANGE']
	diameter_top = params['DIAMETER_TOP']
	total_length = params['TOTAL_LENGTH']
	length_guide = params['LENGTH_GUIDE']
	length_out_of_plate = params['LENGTH_OUT_OF_PLATE']
	length_flange = params['LENGTH_FLANGE']
	length_insertion = params['LENGTH_INSERTION']
	radius1 = params['RADIUS1']
	radius2 = params['RADIUS2']
	radius3 = params['RADIUS3']
	chamfer = params['CHAMFER']
	length_cavity = params['LENGTH_CAVITY']
	thickness_plate = params['THICKNESS_PLATE']
	inv_element = params['INV_ELEMENT']
	add_length_cavity = params['ADD_LENGTH_CAVITY']

	Uni_list = []
	Uni_list2 = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	#Invert Element
	if inv_element:
		VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, thickness_plate, start_pos)
		inv_dir.Set(-inv_dir.Cx(), -inv_dir.Cy(), -inv_dir.Cz())
	#Create Cavity	
	cyl1 = VISISolidFactory.CreateCylinder(diameter_guide/2, thickness_plate+add_length_cavity, start_pos, x_axis, inv_dir)	
	cam_list = [diameter_guide/2, thickness_plate+add_length_cavity]
	VISI.AttachCamAttributes(cyl1,cam_list,"BGB5_1",elem_tag,params["BUILDER_UNIT"])
	# # Alternative for Cavity
	# if thickness_plate<length_cavity:
		# cyl1 = VISISolidFactory.CreateCylinder(diameter_guide/2, thickness_plate+add_length_cavity, start_pos, x_axis, inv_dir)	
		# cam_list = [diameter_guide/2, thickness_plate+add_length_cavity]
		# VISI.AttachCamAttributes(cyl1,cam_list,"BGB5_1",elem_tag,params["BUILDER_UNIT"])
	# else:
		# cyl1 = VISISolidFactory.CreateCylinder(diameter_guide/2, length_cavity+add_length_cavity, start_pos, x_axis, inv_dir)	
		# cam_list = [diameter_guide/2, length_cavity+add_length_cavity]
		# VISI.AttachCamAttributes(cyl1,cam_list,"BGB5_1",elem_tag,params["BUILDER_UNIT"])
	
	## return the tag of created cavity, 0 if no cavity
	
	return cyl1

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_bgb5'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	ref_direction = params['REF_DIRECTION']
	Xaxis.Set(ref_direction.Cx(),ref_direction.Cy(),ref_direction.Cz())

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
