import VISI, VISIParam, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib, math
variables = 0
params = 0
key_name = 0

def CreateElement ():
    position = params['POSITION']
    direction = params['DIRECTION']
    start_plate = params['START_PLATE']
    end_plate = params['END_PLATE']
    start_thick = params['START_THICK']
    end_thick = params['END_THICK']
    total_thick = params['TOTAL_THICK']
    diameter = params['DIAMETER']
    head_diameter = params['HEAD_DIAMETER']
    head_height = params['HEAD_HEIGHT']
    under_head_rad = params['UNDER_HEAD_RAD']
    min_length = params['MIN_LENGTH']
    length = params['LENGTH']
    head_clearance = params['HEAD_CLEARANCE']
    body_clearance = params['BODY_CLEARANCE']
    body_firstplate_clearance = params['BODY_FIRSTPLATE_CLEARANCE']
    clearance_under_head = params['CLEARANCE_UNDER_HEAD']
    guide_factor = params['GUIDE_FACTOR']
    guided_length = params['GUIDED_LENGTH']
    head_without_housing = params['HEAD_WITHOUT_HOUSING']
    follow_uhclearance = params['FOLLOW_UHCLEARANCE']
    extra_length = params['EXTRA_LENGTH']
    shaped_ext = params['SHAPED_EXT']
    expulsion = params['EXPULSION']
    height_list = params['HEIGHT_LIST']
    total_thick_expulsion = params['TOTAL_THICK_EXPULSION']
    height_list_val = params['HEIGHT_LIST_VAL']

    Uni_list = []
    Sub_list = []
    Del_list = []
    Result_List = []
    face_list = []
    start_pos = VISIPoint.New()
    start_pos.Set(position.x(), position.y(), position.z())
    x_axis = VISIVector.New()
    VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
    inv_dir = VISIVector.New()
    inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())

    if ( length >= min_length ):
        extra_thick = length - min_length
    else:
        extra_thick = min_length - length
    if(follow_uhclearance):
        VISIGeoLib.ProjectPointOntoVector(start_pos, direction, clearance_under_head, start_pos)	
    
    ## Creating Head
    if(head_without_housing):
	head = VISISolidFactory.CreateCylinder(head_diameter / 2, head_height, start_pos, x_axis, inv_dir)
	Uni_list.append(head)
    else:
	head = VISISolidFactory.CreateCylinder(head_diameter / 2, head_height, start_pos, x_axis, direction)
	Uni_list.append(head)
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, head_height, start_pos)

    ## body: body to adapt with the end plate
    ## min_body: ejector cylinder until end plate 
    ## Creating Body
    
    if(follow_uhclearance):
        head_hole = head_height + clearance_under_head
    else:
        if(head_without_housing):
            head_hole = 0;
        else:
            head_hole = head_height
    if(expulsion):
        body_height = total_thick_expulsion + height_list_val + extra_length - head_hole
        adapt = 0
        min_body_height = 0 ## means no adaption
    else:
        if ( shaped_ext  or (extra_length == 0) ):  ## the extrusion is obteined moving the faces
            body_height = total_thick + extra_thick - head_hole
            adapt = 1
            min_body_height = total_thick - end_thick/10*9 - head_hole
        else:
            body_height = total_thick + extra_length - head_hole
            adapt = 0
            min_body_height = 0 ## means no adaption

    body = VISISolidFactory.CreateCylinder(diameter / 2, body_height, start_pos, x_axis, direction)
    min_body = 0
    if ( adapt):
        if ( min_body_height > 0 ):
            min_body = VISISolidFactory.CreateCylinder(diameter / 2, min_body_height, start_pos, x_axis, direction)
        body= VISIParam.AdaptCavityToPlate(end_plate, body, 0, face_list)
        Uni_list.append(body)
        if ( min_body):
            Sub_list.append(min_body)
            copy_body = VISISolidFactory.Copy(body)
            Sub_list.append(copy_body)
            VISISolidFactory.Subtract(Sub_list, Result_List)
            for tmp_body in Result_List:
                tmp_face = VISISolidFactory.GetFaceInBodyFromPoint(tmp_body, start_pos)
                if ( tmp_face ):
                    Uni_list.append(tmp_body)
                else:
                    Del_list.append(tmp_body)
            if (len(Del_list)):
                VISISolidFactory.DeleteBodies(Del_list)
    else:
        Uni_list.append(body)
    ## Unite
    Result_List = []
    VISISolidFactory.Unite(Uni_list, Result_List)
    if ( len(face_list) and shaped_ext  and  extra_length):
        VISISolidFactory.ExtrudeFaces(face_list, direction, extra_length)

    if(under_head_rad > 0):
       ## Blend
		if extra_length:
			if shaped_ext:
				edges= [24]
			else:
				edges = [19]
		else :
			edges = [24]
		VISISolidFactory.BlendEdges(Result_List[0], edges, under_head_rad, 1)
                    
    return Result_List[0]

def CreateCavity (elem_tag):
    position = params['POSITION']
    direction = params['DIRECTION']
    start_plate = params['START_PLATE']
    end_plate = params['END_PLATE']
    start_thick = params['START_THICK']
    end_thick = params['END_THICK']
    total_thick = params['TOTAL_THICK']
    diameter = params['DIAMETER']
    head_diameter = params['HEAD_DIAMETER']
    head_height = params['HEAD_HEIGHT']
    under_head_rad = params['UNDER_HEAD_RAD']
    min_length = params['MIN_LENGTH']
    length = params['LENGTH']
    head_clearance = params['HEAD_CLEARANCE']
    body_clearance = params['BODY_CLEARANCE']
    body_firstplate_clearance = params['BODY_FIRSTPLATE_CLEARANCE']
    clearance_under_head = params['CLEARANCE_UNDER_HEAD']
    guide_factor = params['GUIDE_FACTOR']
    guided_length = params['GUIDED_LENGTH']
    head_without_housing = params['HEAD_WITHOUT_HOUSING']
    follow_uhclearance = params['FOLLOW_UHCLEARANCE']
    calc_guided_length = params['CALC_GUIDED_LENGHT']
    extra_length = params['EXTRA_LENGTH']
    shaped_ext = params['SHAPED_EXT']
    expulsion = params['EXPULSION']
    height_list = params['HEIGHT_LIST']
    total_thick_expulsion = params['TOTAL_THICK_EXPULSION']
    height_list_val = params['HEIGHT_LIST_VAL']

    face_list = []
    Uni_list = []
    Result_List = []
    start_pos = VISIPoint.New()
    start_pos.Set(position.x(), position.y(), position.z())
    x_axis = VISIVector.New()
    VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
    inv_dir = VISIVector.New()
    inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())

    if ( length >= min_length ):
        extra_thick = length - min_length
    else:
        extra_thick = min_length - length

    if ( extra_length < 0):
        extra_length = 0
		
    if(head_without_housing):
        VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, head_height, start_pos)
        clearance_under_head = 0  ## clearance_under_head must be null with : head_without_housing
    else:
        cav1 = VISISolidFactory.CreateCylinder((head_diameter + head_clearance)/2, head_height + clearance_under_head,
                                           start_pos, x_axis, direction)
        cam_list = [(head_diameter + head_clearance), (head_height + clearance_under_head)]
        VISI.AttachCamAttributes(cav1,cam_list,"CP1_1",elem_tag,params["BUILDER_UNIT"])
        Uni_list.append(cav1)

    VISIGeoLib.ProjectPointOntoVector(start_pos, direction, head_height + clearance_under_head + under_head_rad, start_pos)
    if(under_head_rad > 0):
        ##Cone
        cone_angle = math.atan( (((diameter + body_firstplate_clearance)/2 + under_head_rad) - (diameter + body_firstplate_clearance)/2) / under_head_rad)
        cone = VISISolidFactory.CreateCone((diameter + body_firstplate_clearance)/2, under_head_rad, cone_angle,
                                           start_pos, x_axis, inv_dir)
        Uni_list.append(cone)

    if(start_plate == end_plate):
        if(head_without_housing):
            cav_h = total_thick  - (clearance_under_head + under_head_rad + calc_guided_length)
        else:
            cav_h = total_thick  - (head_height + clearance_under_head + under_head_rad + calc_guided_length)
            
        cav2 = VISISolidFactory.CreateCylinder((diameter + body_firstplate_clearance)/2,cav_h,
                                       start_pos, x_axis, direction)
        cam_list = [(diameter + body_firstplate_clearance), cav_h]
        VISI.AttachCamAttributes(cav2,cam_list,"CP1_2",elem_tag,params["BUILDER_UNIT"])
        Uni_list.append(cav2)
        VISIGeoLib.ProjectPointOntoVector(start_pos, direction, cav_h , start_pos)
    else:	
	if(head_without_housing):
	    cav_h = start_thick - (clearance_under_head + under_head_rad)
	else:
	    cav_h = start_thick - (head_height + clearance_under_head + under_head_rad)
	    
	cav2 = VISISolidFactory.CreateCylinder((diameter + body_firstplate_clearance)/2, cav_h,
					       start_pos, x_axis, direction)
	cam_list = [(diameter + body_firstplate_clearance), cav_h]
	VISI.AttachCamAttributes(cav2,cam_list,"CP1_2",elem_tag,params["BUILDER_UNIT"])
	Uni_list.append(cav2)
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, cav_h, start_pos)

	if(expulsion and height_list_val):
            cav_h = total_thick_expulsion - (start_thick + calc_guided_length)
        else:
            cav_h = total_thick - (start_thick + calc_guided_length)
            
        cav3 = VISISolidFactory.CreateCylinder((diameter + body_clearance)/2,
                                                           cav_h, start_pos, x_axis, direction)
        cam_list = [(diameter + body_clearance), cav_h]
        VISI.AttachCamAttributes(cav3,cam_list,"CP1_3",elem_tag,params["BUILDER_UNIT"])
        Uni_list.append(cav3)
        VISIGeoLib.ProjectPointOntoVector(start_pos, direction, cav_h, start_pos)

    if(expulsion):
        cav_h = calc_guided_length + height_list_val + extra_length
        adapt = 0 ## means no adaption
    else:
        if ( shaped_ext  or (extra_length == 0) ):  ## the extrusion is obteined moving the faces
            cav_h = calc_guided_length + extra_thick
            adapt = 1
        else:
            cav_h = calc_guided_length + extra_length
            adapt = 0 ## means no adaption

    if(calc_guided_length > 0):       
        cav4 = VISISolidFactory.CreateCylinder(diameter/2, cav_h, start_pos, x_axis, direction)
        cam_list = [diameter, calc_guided_length]
        VISI.AttachCamAttributes(cav4,cam_list,"CP1_4",elem_tag,params["BUILDER_UNIT"])
        if ( adapt):
            cav4= VISIParam.AdaptCavityToPlate(end_plate, cav4, 0, face_list)
        
        Uni_list.append(cav4)
                
    ##Unite The Sprue Bushing Parts 
    VISISolidFactory.Unite(Uni_list, Result_List,0)
    if ( len(face_list) and shaped_ext  and  extra_length):
        VISISolidFactory.ExtrudeFaces(face_list, direction, extra_length)
    return Result_List[0]

def Execute (names, dictionaries):
    global variables
    global params
    global key_name
    param_key = ''
    if(key_name == 0):
	param_key = 'PARAM_NG_cp1'
    else:
	param_key = key_name
    params = variables[param_key]


    position = params['POSITION']
    save_pos = VISIPoint.New()
    save_pos.Set(position.x(),position.y(),position.z())
    direction = params['DIRECTION']
    save_dir = VISIVector.New()
    save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
    Xaxis = VISIVector.New()
    VISIGeoLib.ArbitraryNormalAxis(direction, Xaxis)

    elem_tag = 0
    if ( params['BUILD_ELEM'] ):
        elem_tag = CreateElement()
    cavity_tag = 0
    if ( params['BUILD_CAVITY'] ):
        cavity_tag = CreateCavity(elem_tag)

    if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
	names.append(param_key)

        elem_info = {}
        elem_info['ELEM_TAG'] = elem_tag
        elem_info['CAVITY_TAG'] = cavity_tag
        elem_info['POSITION'] = save_pos
        elem_info['DIRECTION'] = save_dir
        elem_info['REF_DIRECTION'] = Xaxis

        dictionaries.append(elem_info)
