import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	diameter = params['DIAMETER']
	diameter_through = params['DIAMETER_THROUGH']
	length = params['LENGTH']
	diameter_head = params['DIAMETER_HEAD']
	head_height = params['HEAD_HEIGHT']
	sink_depth = params['SINK_DEPTH']
	clear_diameter = params['CLEAR_DIAMETER']

	Uni_list = []
	Uni_list2 = []
	Uni_list3 = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	#Adjust Depth
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, sink_depth, start_pos)
	
	#Create Body1
	cyl0 = VISISolidFactory.CreateCylinder(diameter/2, length/2, start_pos, x_axis, inv_dir)
	Uni_list.append(cyl0)
	
	#Chamfer
	edges = [12]
	VISISolidFactory.ChamferEdges(cyl0, edges,diameter/50, diameter/50 , 1)
	edges = [6]
	VISISolidFactory.ChamferEdges(cyl0, edges,diameter/30, diameter/30 , 1)
	
	#Create Body2
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, length/2, start_pos)
	cyl1 = VISISolidFactory.CreateCylinder(diameter/2, length/2, start_pos, x_axis, inv_dir)
	Uni_list.append(cyl1)
	
	#Chamfer
	edges = [6]
	VISISolidFactory.ChamferEdges(cyl1, edges,diameter/50, diameter/50 , 1)
	edges = [12]
	VISISolidFactory.ChamferEdges(cyl1, edges,diameter/30, diameter/30 , 1)
	
	#Unite
	VISISolidFactory.Unite(Uni_list, Uni_list2)
	
	#Create Head
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, length/2, start_pos)
	cyl2 = VISISolidFactory.CreateCylinder(diameter_head/2, head_height, start_pos, x_axis, inv_dir)
	Uni_list2.append(cyl2)
	
	#Create Through Hole
	cyl3 = VISISolidFactory.CreateCylinder(diameter_through/2, length, start_pos, x_axis, inv_dir)
	Uni_list2.append(cyl3)
	
	#Subtract
	VISISolidFactory.Subtract(Uni_list2, Result_List)
	
	return Result_List[0]

def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	diameter = params['DIAMETER']
	diameter_through = params['DIAMETER_THROUGH']
	length = params['LENGTH']
	diameter_head = params['DIAMETER_HEAD']
	head_height = params['HEAD_HEIGHT']
	sink_depth = params['SINK_DEPTH']
	clear_diameter = params['CLEAR_DIAMETER']
	
	Uni_list = []
	Uni_list2 = []
	Uni_list3 = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	#Adjust length
	#VISIGeoLib.ProjectPointOntoVector(start_pos, direction, length-internal_length, start_pos)
	
	#Create Body1
	cyl0 = VISISolidFactory.CreateCylinder(diameter/2+clear_diameter/2, length+sink_depth, start_pos, x_axis, inv_dir)

	cam_list = [diameter+clear_diameter, length+sink_depth]
	VISI.AttachCamAttributes(cyl0,cam_list,"CSC1_1",elem_tag,params["BUILDER_UNIT"])
	
	## return the tag of created cavity, 0 if no cavity
	return cyl0

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_csc1'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	ref_direction = params['REF_DIRECTION']
	Xaxis.Set(ref_direction.Cx(),ref_direction.Cy(),ref_direction.Cz())

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
