import VISI, VISIParam, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib, math
variables = 0
params = 0
key_name = 0

def CreateElement ():
    position = params['POSITION']
    direction = params['DIRECTION']
    start_plate = params['START_PLATE']
    end_plate = params['END_PLATE']
    start_thick = params['START_THICK']
    end_thick = params['END_THICK']
    total_thick = params['TOTAL_THICK']
    sel_pos = params['SEL_POS']
    diameter = params['DIAMETER']
    head_diameter = params['HEAD_DIAMETER']
    head_height = params['HEAD_HEIGHT']
    length = params['LENGTH']
    min_length = params['MIN_LENGTH']
    head_diameter_clearance = params['HEAD_DIAMETER_CLEARANCE']
    clearance_under_head = params['CLEARANCE_UNDER_HEAD']
    stem_diameter_clearance = params['STEM_DIAMETER_CLEARANCE']
    clearance_first_plate = params['CLEARANCE_FIRST_PLATE']
    guide_factor = params['GUIDE_FACTOR']
    guiding_ejector_length = params['GUIDING_EJECTOR_LENGTH']
    head_without_housing = params['HEAD_WITHOUT_HOUSING']
    follow_clearance_under_head = params['FOLLOW_CLEARANCE_UNDER_HEAD']
    total_head_height = params['TOTAL_HEAD_HEIGHT']
    extra_length = params['EXTRA_LENGTH']
    shaped_ext = params['SHAPED_EXT']
    under_head_rad = params['UNDER_HEAD_RAD']
    
    tmp_pos = VISIPoint.New()
    x_axis = VISIVector.New()
    rev_dir = VISIVector.New()
    Bool_List = []
    Result_List = []
    face_list = []
    my_list = []


    VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
    rev_dir.Set(- direction.Cx(), - direction.Cy(), - direction.Cz())

    if ( length >= min_length ):
	extra_thick = length - min_length
    else:
	extra_thick = min_length - length
    ## calculate the start point to build the ejector
    if (head_without_housing):
	VISIGeoLib.ProjectPointOntoVector(position, direction, - total_head_height, tmp_pos)
	cyl_h = total_thick
    else:
	if (follow_clearance_under_head):
	    VISIGeoLib.ProjectPointOntoVector(position, direction, clearance_under_head, tmp_pos)
	    cyl_h = total_thick - clearance_under_head - total_head_height
	else:
	    tmp_pos.Set(position.x(), position.y(), position.z())
	    cyl_h = total_thick - total_head_height

    ## create the cylinder of the ejector head
    cyl_head = VISISolidFactory.CreateCylinder(head_diameter / 2, head_height, tmp_pos, x_axis, direction)
    Bool_List.append(cyl_head)
    VISIGeoLib.ProjectPointOntoVector(tmp_pos, direction, total_head_height, tmp_pos)
    ## create the cone of the ejector head
    cone_head = VISISolidFactory.CreateCone(diameter / 2, total_head_height - head_height,
					    math.pi / 6, tmp_pos, x_axis, rev_dir)
    Bool_List.append(cone_head)

    min_body_h = 0
    if(extra_length == 0):
	body_h = cyl_h + extra_thick
	min_body_h = cyl_h - end_thick
	adapt = 1
    else:
	if ( shaped_ext ):
	    body_h = cyl_h + extra_thick
	    min_body_h = cyl_h - end_thick
	    adapt = 1
	else:
	    body_h = cyl_h + extra_length
	    adapt = 0
		    
    ## create the body of the ejector
    body = VISISolidFactory.CreateCylinder(diameter / 2, body_h, tmp_pos, x_axis, direction)
    if ( adapt):
	if ( min_body_h > 0):
		min_body = VISISolidFactory.CreateCylinder(diameter / 2, min_body_h, tmp_pos, x_axis, direction)
		Bool_List.append(min_body)
		my_list.append(min_body)
	body= VISIParam.AdaptCavityToPlate(end_plate, body, 0, face_list)

    my_list.append(cyl_head)
    my_list.append(cone_head)
    VISISolidFactory.Unite(my_list, Result_List)
    part1 = Result_List[0]
	
    
    Result_List = []
    Bool_List= []
    Bool_List.append(body)
    Bool_List.append(part1)
    #return min_body
    
    ## unite all parts of the ejector
    VISISolidFactory.Unite(Bool_List, Result_List)
    if ( len(face_list) and shaped_ext  and  extra_length):
        VISISolidFactory.ExtrudeFaces(face_list, direction, extra_length)

    if(under_head_rad > 0):
        ## Blend
        if extra_length:
            if shaped_ext:
                edges = [81]
            else:
                edges = [21]
        else:
            edges = [81]
    VISISolidFactory.BlendEdges(Result_List[0], edges,under_head_rad, 1)
    
    ##VISI.MessageBox("", str(under_head_rad))
                    
    return Result_List[0]

def CreateCavity (elem_tag):
    position = params['POSITION']
    direction = params['DIRECTION']
    start_plate = params['START_PLATE']
    end_plate = params['END_PLATE']
    start_thick = params['START_THICK']
    end_thick = params['END_THICK']
    total_thick = params['TOTAL_THICK']
    sel_pos = params['SEL_POS']
    diameter = params['DIAMETER']
    head_diameter = params['HEAD_DIAMETER']
    head_height = params['HEAD_HEIGHT']
    length = params['LENGTH']
    min_length = params['MIN_LENGTH']
    head_diameter_clearance = params['HEAD_DIAMETER_CLEARANCE']
    clearance_under_head = params['CLEARANCE_UNDER_HEAD']
    stem_diameter_clearance = params['STEM_DIAMETER_CLEARANCE']
    clearance_first_plate = params['CLEARANCE_FIRST_PLATE']
    guide_factor = params['GUIDE_FACTOR']
    guiding_ejector_length = params['GUIDING_EJECTOR_LENGTH']
    head_without_housing = params['HEAD_WITHOUT_HOUSING']
    follow_clearance_under_head = params['FOLLOW_CLEARANCE_UNDER_HEAD']
    total_head_height = params['TOTAL_HEAD_HEIGHT']
    calc_guided_length = params['CALC_GUIDED_LENGHT']
    extra_length = params['EXTRA_LENGTH']
    shaped_ext = params['SHAPED_EXT']


    tmp_pos = VISIPoint.New()
    start_pos = VISIPoint.New()
    x_axis = VISIVector.New()
    rev_dir = VISIVector.New()
    Bool_List = []
    Result_List = []
    face_list = []
	
    VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
    rev_dir.Set(- direction.Cx(), - direction.Cy(), - direction.Cz())

    if ( length >= min_length ):
	extra_thick = length - min_length
    else:
	extra_thick = min_length - length

    if ( extra_length < 0):
        extra_length = 0
		
    ## build cavity in case of head without housing
    if (head_without_housing):
	start_pos.Set(position.x(), position.y(), position.z())
	head_cav_height = 0
    else:
	if (start_plate == end_plate):
	    clearance_first_plate = stem_diameter_clearance	
	base_cone = (clearance_under_head + total_head_height) * math.tan(math.pi / 6)
	pre_drill_cone_height = ((head_diameter + head_diameter_clearance - (diameter +clearance_first_plate)) / 2) / math.tan(math.pi / 6)
	pre_drill_height = clearance_under_head + total_head_height - pre_drill_cone_height
	head_cav_height = pre_drill_height + pre_drill_cone_height
		    ## create the ejector cavity pre drill
	if ( pre_drill_height > (clearance_first_plate/2 / math.tan(math.pi / 6)) ):
	    cav = VISISolidFactory.CreateCylinder((head_diameter + head_diameter_clearance) / 2, pre_drill_height - (clearance_first_plate/2 / math.tan(math.pi / 6)),
								  position, x_axis, direction)
	    cam_list = [(head_diameter + head_diameter_clearance) , pre_drill_height]
	    VISI.AttachCamAttributes(cav,cam_list,"EP4_2",elem_tag,params["BUILDER_UNIT"])
	    Bool_List.append(cav)

	    VISIGeoLib.ProjectPointOntoVector(position, direction,
						      pre_drill_height + pre_drill_cone_height - (clearance_first_plate/2 / math.tan(math.pi / 6)), tmp_pos)
	
	    ## create the countersink of the cavity
	    cav = VISISolidFactory.CreateCone((diameter + clearance_first_plate)/ 2, pre_drill_cone_height, math.pi / 6,
					      tmp_pos, x_axis, rev_dir)
	    cam_list = [(head_diameter + head_diameter_clearance) , pre_drill_cone_height]
	    VISI.AttachCamAttributes(cav,cam_list,"EP4_1",elem_tag,params["BUILDER_UNIT"])
	    Bool_List.append(cav)
	else:
	    VISIGeoLib.ProjectPointOntoVector(position, direction,
						      pre_drill_height + pre_drill_cone_height, tmp_pos)
	    
	    ## create the countersink of the cavity
	    cav = VISISolidFactory.CreateCone((diameter + clearance_first_plate)/ 2, head_cav_height, math.pi / 6,
					      tmp_pos, x_axis, rev_dir)
	    cam_list = [(head_diameter + head_diameter_clearance) , head_cav_height]
	    VISI.AttachCamAttributes(cav,cam_list,"EP4_1",elem_tag,params["BUILDER_UNIT"])
	    Bool_List.append(cav)

	VISIGeoLib.ProjectPointOntoVector(position, direction, head_cav_height - (clearance_first_plate/2 / math.tan(math.pi / 6)), start_pos)

    if (start_plate == end_plate):
	cav3_h = 0
	cav4_h = total_thick  - calc_guided_length
    else:
        if (head_without_housing):
            cav3_h = start_thick - head_cav_height
        else:    
            cav3_h = start_thick - head_cav_height + (clearance_first_plate/2 / math.tan(math.pi / 6))
	cav4_h = total_thick  - start_thick - calc_guided_length

    if(extra_length == 0):
	cav5_h = calc_guided_length + extra_thick
	if (start_plate == end_plate):
	    min_body_h = 0
	else:
	    min_body_h = calc_guided_length - end_thick
	adapt = 1
    else:
	if ( shaped_ext ):
	    cav5_h = calc_guided_length + extra_thick
	    if (start_plate == end_plate):
		min_body_h = 0
	    else:
		min_body_h = calc_guided_length - end_thick
	    adapt = 1
	else:
	    cav5_h = calc_guided_length + extra_length
	    min_body_h = 0
	    adapt = 0

    if ( cav3_h > 0):
	cav = VISISolidFactory.CreateCylinder((diameter + clearance_first_plate) / 2, cav3_h, start_pos, x_axis, direction)
	cam_list = [(diameter + clearance_first_plate) , cav3_h]
	VISI.AttachCamAttributes(cav,cam_list,"EP4_3",elem_tag,params["BUILDER_UNIT"])
	Bool_List.append(cav)
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, cav3_h, start_pos)
		
    cav = VISISolidFactory.CreateCylinder((diameter + stem_diameter_clearance) / 2, cav4_h, start_pos, x_axis, direction)
    cam_list = [(diameter + stem_diameter_clearance), cav4_h]
    VISI.AttachCamAttributes(cav,cam_list,"EP4_4",elem_tag,params["BUILDER_UNIT"])
    Bool_List.append(cav)
    VISIGeoLib.ProjectPointOntoVector(start_pos, direction, cav4_h, start_pos)

    ## create the guided part of the cavity
    cav = VISISolidFactory.CreateCylinder(diameter / 2, cav5_h, start_pos, x_axis, direction)
    cam_list = [diameter , cav5_h]
    VISI.AttachCamAttributes(cav,cam_list,"EP4_5",elem_tag,params["BUILDER_UNIT"])
    if ( adapt):
	if ( min_body_h > 0):
	    min_body = VISISolidFactory.CreateCylinder(diameter / 2, min_body_h, start_pos, x_axis, direction)
	    Bool_List.append(min_body)
	cav= VISIParam.AdaptCavityToPlate(end_plate, cav, 0, face_list)
    Bool_List.append(cav)


    ## unite all parts of the ejector
    VISISolidFactory.Unite(Bool_List, Result_List,0)

    if ( len(face_list) and shaped_ext and extra_length ):
	VISISolidFactory.ExtrudeFaces(face_list, direction, extra_length)
	
			
    ## unite all the ejector cavity parts
    VISISolidFactory.Unite(Bool_List, Result_List,0)
	
    return Result_List[0]

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_ep4'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, Xaxis)

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
