import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib, math, VISIParam
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	thr_diameter = params['THR_DIAMETER']
	diameter = params['DIAMETER']
	ext_diameter = params['EXT_DIAMETER']
	key_diameter = params['KEY_DIAMETER']
	key_height = params['KEY_HEIGHT']
	collar_diameter = params['COLLAR_DIAMETER']
	thr_length = params['THR_LENGTH']
	length = params['LENGTH']
	hex_clear = params['HEX_CLEAR']
	pre_hole_diam = params['PRE_HOLE_DIAM']
	thr_dep_screw = params['THR_DEP_SCREW']
	hole_dep_screw = params['HOLE_DEP_SCREW']
	thread_pitch = params['THREAD_PITCH']
	housing_height = params['HOUSING_HEIGHT']
	
	#direction.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())

	Front_Body_List = []
	Back_Body_List = []
	Uni_list = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())

	if(housing_height):
		VISIGeoLib.ProjectPointOntoVector(start_pos, direction, housing_height, start_pos)

	cyl1 = VISISolidFactory.CreateCylinder(collar_diameter/2, key_height, start_pos, x_axis, inv_dir)
	Uni_list.append(cyl1)
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, key_height, start_pos)
		
	cyl1 = VISISolidFactory.CreateCylinder(ext_diameter/2, length - key_height - thr_length, start_pos, x_axis, inv_dir)
	Uni_list.append(cyl1)

	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, key_height, start_pos)
	cyl2 = VISISolidFactory.CreateCylinder(diameter/2, thr_length, start_pos, x_axis, direction)
	Uni_list.append(cyl2)

	## attach Thread attribute
	VISI.AttachThreadAttributes(cyl2, thread_pitch)

	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, thr_length, start_pos)
	hole_prism = VISISolidFactory.CreatePrism(key_diameter * math.tan(math.pi/6.), length, 6, start_pos, x_axis, inv_dir)

	VISISolidFactory.Unite(Uni_list, Result_List)

	body_list = []
	body_list.append(Result_List[0])
	body_list.append(hole_prism)
	Result_List = []
	VISISolidFactory.Subtract(body_list, Result_List)

	chamfer = (diameter - pre_hole_diam)/2
	if ( chamfer):
		edges = [47]
		VISISolidFactory.ChamferEdges(Result_List[0], edges, chamfer, chamfer, 1)

	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, length, start_pos)
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, 0.003, start_pos)
	VISISolidFactory.CutWithPlane(start_pos, inv_dir, Result_List[0], Front_Body_List, Back_Body_List)
	part1 = Front_Body_List[0]
	part2 = Back_Body_List[0]
	
	edges = [25, 275]
	VISISolidFactory.ChamferEdges(part1, edges, 0.001, 0.001, 1)

	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, 0.001, start_pos)
	Front_Body_List = []
	Back_Body_List = []
	VISISolidFactory.CutWithPlane(start_pos, inv_dir, part2, Front_Body_List, Back_Body_List)
	part3 = Front_Body_List[0]
	part4 = Back_Body_List[0]

	del_list = []
	del_list.append(part3)
	VISISolidFactory.DeleteBodies(del_list)

	edges = [6]
	VISISolidFactory.ChamferEdges(part4, edges, 0.001, 0.001, 1)

	cyl1 = VISISolidFactory.CreateCylinder(ext_diameter/2 - (0.001 * math.tan(math.radians(45))), 0.001, start_pos, x_axis, inv_dir)
	hole_prism1 = VISISolidFactory.CreatePrism(key_diameter * math.tan(math.pi/6.), 0.001, 6, start_pos, x_axis, inv_dir)

	Uni_list = []
	Result_List = []
	Uni_list.append(part1)
	Uni_list.append(part4)
	Uni_list.append(cyl1)
	VISISolidFactory.Unite(Uni_list, Result_List)

	body_list = []
	body_list.append(Result_List[0])
	body_list.append(hole_prism1)
	Result_List = []
	VISISolidFactory.Subtract(body_list, Result_List)

	return Result_List[0]

def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	thr_diameter = params['THR_DIAMETER']
	diameter = params['DIAMETER']
	ext_diameter = params['EXT_DIAMETER']
	key_diameter = params['KEY_DIAMETER']
	key_height = params['KEY_HEIGHT']
	collar_diameter = params['COLLAR_DIAMETER']
	thr_length = params['THR_LENGTH']
	length = params['LENGTH']
	hex_clear = params['HEX_CLEAR']
	pre_hole_diam = params['PRE_HOLE_DIAM']
	thr_dep_screw = params['THR_DEP_SCREW']
	hole_dep_screw = params['HOLE_DEP_SCREW']
	thread_pitch = params['THREAD_PITCH']
	housing_height = params['HOUSING_HEIGHT']
	start_plate = params['START_PLATE']
	hex_cav_diam = params['HEX_CAV_DIAM']
	
	#direction.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	Front_Body_List = []
	Back_Body_List = []
	Uni_list = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())

	pos = VISIPoint.New()
	pos.Set(position.x(), position.y(), position.z())

	if(housing_height):
                value = round( ((key_diameter/2 / math.cos(math.radians(30)))), 4)      
                #cav1 = VISISolidFactory.CreateCylinder((key_diameter + hex_clear)/2, housing_height, start_pos, x_axis, direction)
                cav1 = VISISolidFactory.CreateCylinder(hex_cav_diam/2, housing_height, start_pos, x_axis, direction)
		#cav1 = VISISolidFactory.CreateCylinder( round(((key_diameter/2 / math.cos(math.radians(30))) + hex_clear), 2), housing_height, start_pos, x_axis, direction)
		VISIGeoLib.ProjectPointOntoVector(start_pos, direction, housing_height, start_pos)
		#Uni_list.append(cav1)
		cam_list = [hex_cav_diam, housing_height]
		VISI.AttachCamAttributes(cav1,cam_list,"FIT2_1",elem_tag,params["BUILDER_UNIT"])

		if (start_plate):
                        cavity_face = VISISolidFactory.GetFaceInBodyFromPoint(cav1, pos)
                        hole = VISIParam.ExtendFaceToSolid(start_plate, cavity_face, direction)
                        hole= VISIParam.AdaptCavityToPlate(start_plate, hole, 0)
                        Uni_list.append(hole)
                else:
                        Uni_list.append(cav1)

	cav2 = VISISolidFactory.CreateCylinder(pre_hole_diam/2, thr_length + thr_dep_screw + hole_dep_screw, start_pos, x_axis, direction)
	#Uni_list.append(cav2)
	cam_list = [pre_hole_diam, thr_length + thr_dep_screw, diameter, thr_length + thr_dep_screw, thread_pitch]
        
	#Differentiate between metric and inch thread
	if thr_diameter[0] == "M":
		VISI.AttachCamAttributes(cav2, cam_list, "FIT2_21", thr_diameter, elem_tag, params["BUILDER_UNIT"])
	else:
		VISI.AttachCamAttributes(cav2, cam_list, "FIT2_22", thr_diameter, elem_tag, params["BUILDER_UNIT"])

        if (start_plate) and not(housing_height):
                cavity_face = VISISolidFactory.GetFaceInBodyFromPoint(cav2, pos)
                hole = VISIParam.ExtendFaceToSolid(start_plate, cavity_face, direction)
                hole= VISIParam.AdaptCavityToPlate(start_plate, hole, 0)
                Uni_list.append(hole)
        else:
                Uni_list.append(cav2)
	
	h_cone = pre_hole_diam/2.* math.tan(math.pi/6)
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, thr_length + thr_dep_screw + hole_dep_screw + h_cone, start_pos)
        cone = VISISolidFactory.CreateCone(0, h_cone, math.pi/3,
                                           start_pos, x_axis, inv_dir)
        Uni_list.append(cone)

	VISISolidFactory.Unite(Uni_list, Result_List,0)
	
	return Result_List[0]

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_fit2_1'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	ref_direction = params['REF_DIRECTION']
	Xaxis.Set(ref_direction.Cx(),ref_direction.Cy(),ref_direction.Cz())

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
