import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib, math
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	diameter = params['DIAMETER']
	total_length = params['TOTAL_LENGTH']
	chamfer2 = params['CHAMFER2']
	radius = params['RADIUS']
	angle0 = params['ANGLE0']
	angle3 = params['ANGLE3']
	distance_chamfer = params['DISTANCE_CHAMFER']
	length_cavity = params['LENGTH_CAVITY']
	length_cavity2 = params['LENGTH_CAVITY2']
	inv_direction = params['INV_DIRECTION']
	ins_depth = params['INS_DEPTH']
	thickness_startplate = params['THICKNESS_STARTPLATE']
	support = params['SUPPORT']
	
	Uni_list = []
	Uni_list2 = []
	Result_List = []
	Result_List2 = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	#set right position
	if support:
		VISIGeoLib.ProjectPointOntoVector(start_pos, direction, thickness_startplate, start_pos)
	else:
		VISIGeoLib.ProjectPointOntoVector(start_pos, direction, ins_depth, start_pos)
	
	#Body
	cyl2 = VISISolidFactory.CreateCylinder(diameter/2, total_length, start_pos, x_axis, direction)
	
	
	#Chamfer
	edges = [6]
	chamfer_up = math.tan(angle0)*chamfer2
	VISISolidFactory.ChamferEdges(cyl2, edges, chamfer2, chamfer_up, 1)
	edges = [12]
	chamfer_up = math.tan(angle3)*distance_chamfer
	VISISolidFactory.ChamferEdges(cyl2, edges, distance_chamfer,chamfer_up, 1)	
	
	#Blend Edges
	edges = [49]
	VISISolidFactory.BlendEdges(cyl2, edges, radius, 1)
	
	return cyl2
	

	
def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	diameter = params['DIAMETER']
	total_length = params['TOTAL_LENGTH']
	chamfer2 = params['CHAMFER2']
	radius = params['RADIUS']
	angle0 = params['ANGLE0']
	angle3 = params['ANGLE3']
	distance_chamfer = params['DISTANCE_CHAMFER']
	length_cavity = params['LENGTH_CAVITY']
	length_cavity2 = params['LENGTH_CAVITY2']
	inv_direction = params['INV_DIRECTION']
	clearance_guide_diameter = params['CLEARANCE_GUIDE_DIAMETER']
	clearance_body_diameter = params['CLEARANCE_BODY_DIAMETER']
	ins_depth = params['INS_DEPTH']
	thickness_startplate = params['THICKNESS_STARTPLATE']
	thickness_endplate = params['THICKNESS_ENDPLATE']
	distance_between_plates = params['DISTANCE_BETWEEN_PLATES']
	support = params['SUPPORT']
	
	Uni_list = []
	Uni_list2 = []
	Result_List = []
	Result_List2 = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	if not support:
		#upper cavity
		cyl1 = VISISolidFactory.CreateCylinder((diameter + clearance_guide_diameter)/2,thickness_startplate, start_pos, x_axis, direction)
		Result_List.append(cyl1)
		cam_list = [diameter + clearance_guide_diameter, thickness_startplate]
		VISI.AttachCamAttributes(cyl1,cam_list,"GP2_1_2",elem_tag,params["BUILDER_UNIT"])
	
	#move to position
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, thickness_startplate, start_pos)
	
	#lower cavity
	cyl2 = VISISolidFactory.CreateCylinder((diameter + clearance_body_diameter)/2,thickness_endplate+distance_between_plates, start_pos, x_axis, direction)
	Result_List.append(cyl2)
	cam_list = [diameter + clearance_body_diameter, thickness_endplate+distance_between_plates]
	VISI.AttachCamAttributes(cyl2,cam_list,"GP2_1_1",elem_tag,params["BUILDER_UNIT"])
	
	
	#Unite
	VISISolidFactory.Unite(Result_List, Result_List2, 0)
	
	return Result_List2[0]

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_gp2_1'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	ref_direction = params['REF_DIRECTION']
	Xaxis.Set(ref_direction.Cx(),ref_direction.Cy(),ref_direction.Cz())

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
