import VISI, VISIParam, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib, math
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	start_plate = params['START_PLATE']
	start_face = params['START_FACE']
	sel_pos = params['SEL_POS']
	face_normal = params['FACE_NORMAL']
	start_thick = params['START_THICK']
	diameter = params['DIAMETER']
	sphere_projection = params['SPHERE_PROJECTION']
	sphere_stroke = params['SPHERE_STROKE']
	sphere_diameter = params['SPHERE_DIAMETER']
	length = params['LENGTH']
	thread_diameter = params['THREAD_DIAMETER']
	thread_pitch = params['THREAD_PITCH']
	housing_height = params['HOUSING_HEIGHT']
	not_housed = params['NOT_HOUSED']
	reverse_element = params['REVERSE_ELEMENT']
	sphere_pos = params['SPHERE_POS']

	start_pos = VISIPoint.New()
	x_axis = VISIVector.New()
	z_axis = VISIVector.New()
	Bool_List = []
	Result_List = []

	## calculate x axis
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)

        ##calculate z axis
	if (reverse_element):
		z_axis.Set(direction.Cx(), direction.Cy(), direction.Cz())
		start_pos.Set(position.x(), position.y(), position.z())
	else:
		z_axis.Set(- direction.Cx(), - direction.Cy(), - direction.Cz())
		VISIGeoLib.ProjectPointOntoVector(position, direction, start_thick, start_pos)
        
        
	## project the position in start point
	## according to the housing check
	if (not_housed):
		VISIGeoLib.ProjectPointOntoVector(start_pos, z_axis, start_thick - housing_height, start_pos)
		z_axis.Set(direction.Cx(), direction.Cy(), direction.Cz())		
	else:
		VISIGeoLib.ProjectPointOntoVector(start_pos, z_axis, start_thick - length - housing_height, start_pos)

	## create the grub
	grub = VISISolidFactory.CreateCylinder(diameter / 2, length, start_pos, x_axis, z_axis)
	Bool_List.append(grub)

	## attach Thread attribute
	VISI.AttachThreadAttributes(grub, thread_pitch)

	## calculate the chamfer values
	chamfer = (diameter - thread_diameter) / 2

	## apply the chamfer superior part of the grub
	if (chamfer):
		edges = [6, 12]
		VISISolidFactory.ChamferEdges(grub, edges, chamfer, chamfer, 1)

        if (sphere_projection > sphere_diameter / 2):
                VISIGeoLib.ProjectPointOntoVector(start_pos, z_axis, length - sphere_pos, start_pos)
                cyl = VISISolidFactory.CreateCylinder(sphere_diameter / 2, sphere_projection - sphere_diameter / 2, start_pos, x_axis, z_axis)
                Bool_List.append(cyl)

                ## create the grub sphere
                VISIGeoLib.ProjectPointOntoVector(start_pos, z_axis, sphere_projection - sphere_diameter / 2, start_pos)
                sphere = VISISolidFactory.CreateSphere(sphere_diameter / 2, start_pos, x_axis, z_axis)
                Bool_List.append(sphere)          
	else:
                VISIGeoLib.ProjectPointOntoVector(start_pos, z_axis, length - (sphere_diameter / 2 - sphere_projection + sphere_pos), start_pos)

                ## create the grub sphere
                sphere = VISISolidFactory.CreateSphere(sphere_diameter / 2, start_pos, x_axis, z_axis)
                Bool_List.append(sphere)

	## unite the sphere to the grub
	VISISolidFactory.Unite(Bool_List, Result_List)
	
	return Result_List[0]

def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	start_plate = params['START_PLATE']
	start_face = params['START_FACE']
	sel_pos = params['SEL_POS']
	face_normal = params['FACE_NORMAL']
	start_thick = params['START_THICK']
	diameter = params['DIAMETER']
	sphere_projection = params['SPHERE_PROJECTION']
	sphere_stroke = params['SPHERE_STROKE']
	sphere_diameter = params['SPHERE_DIAMETER']
	length = params['LENGTH']
	thread_diameter = params['THREAD_DIAMETER']
	thread_pitch = params['THREAD_PITCH']
	housing_height = params['HOUSING_HEIGHT']
	not_housed = params['NOT_HOUSED']
	thr_dep_screw = params['THR_DEP_SCREW']
	hole_dep_screw = params['HOLE_DEP_SCREW']
	through_hole = params['THROUGH_HOLE']
	no_through_thread = params['NO_THROUGH_THREAD']
	reverse_element = params['REVERSE_ELEMENT']

	body_list = []
	result_list = []
	point_list = []
	pos_copy1 = VISIPoint.New()
	pos_copy2 = VISIPoint.New()
	x_axis = VISIVector.New()
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	dir = VISIVector.New()
	dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())

	## calculate x axis
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)

	if (reverse_element):
		direction.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
		VISIGeoLib.ProjectPointOntoVector(position, direction, -start_thick, start_pos)
	
	
	if (no_through_thread):
		threaded_height = length + thr_dep_screw
		cav = VISISolidFactory.CreateCylinder(thread_diameter / 2, start_thick, start_pos, x_axis, direction)
		cam_list = [thread_diameter, start_thick, diameter, threaded_height, thread_pitch]
		VISI.AttachCamAttributes(cav, cam_list, "GS4_1", elem_tag, params["BUILDER_UNIT"])
		
	else:
		cav = VISISolidFactory.CreateCylinder(thread_diameter / 2, start_thick, start_pos, x_axis, direction)
		cam_list = [thread_diameter, start_thick, diameter, start_thick, thread_pitch]
		VISI.AttachCamAttributes(cav,cam_list,"GS4_1",elem_tag,params["BUILDER_UNIT"])

	return cav

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_gs4_1'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, Xaxis)

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
