import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib, math
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	diameter = params['DIAMETER']
	diameter_hole = params['DIAMETER_HOLE']
	diameter_cone = params['DIAMETER_CONE']
	height = params['HEIGHT']
	chamfer = params['CHAMFER']
	angle0 = params['ANGLE0']
	angle1 = params['ANGLE1']
	screw_diameter = params['SCREW_DIAMETER']
	ins_depth = params['INS_DEPTH']

	Uni_list = []
	Uni_list2 = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	#Insdepth
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, ins_depth, start_pos)	
	
	
	#Create Body
	cyl1 = VISISolidFactory.CreateCylinder(diameter/2, height, start_pos, x_axis, direction)
	Uni_list.append(cyl1)
	
	#Chamfer
	edges = [6,12]
	VISISolidFactory.ChamferEdges(cyl1, edges, chamfer, chamfer, 1)
	
	#Create Through hole
	cyl2 = VISISolidFactory.CreateCylinder(diameter_hole/2, height, start_pos, x_axis, direction)
	Uni_list.append(cyl2)
	
	#Create Cone
	h_cone = (diameter_cone/2-diameter_hole/2)/(math.tan(angle1/2))
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, height - h_cone, start_pos)	
	cone = VISISolidFactory.CreateCone(diameter_hole/2, h_cone, angle1/2, start_pos, x_axis, direction)
	Uni_list.append(cone)
	
	#Subtract
	VISISolidFactory.Subtract(Uni_list, Result_List)
	
	return Result_List[0]

def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	diameter = params['DIAMETER']
	diameter_hole = params['DIAMETER_HOLE']
	height = params['HEIGHT']
	chamfer = params['CHAMFER']
	angle0 = params['ANGLE0']
	angle1 = params['ANGLE1']
	screw_diameter = params['SCREW_DIAMETER']
	play_diameter = params['PLAY_DIAMETER']
	play_height = params['PLAY_HEIGHT']
	ins_depth = params['INS_DEPTH']
	
	Uni_list = []
	Uni_list2 = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	#Insdepth
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, ins_depth, start_pos)	
	
	#Create Body
	cyl1 = VISISolidFactory.CreateCylinder(diameter/2+play_diameter/2, height+play_height, start_pos, x_axis, direction)
	Uni_list.append(cyl1)

	return cyl1

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_hd1'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	ref_direction = params['REF_DIRECTION']
	Xaxis.Set(ref_direction.Cx(),ref_direction.Cy(),ref_direction.Cz())

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
