import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib, math
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	sel_pos = params['SEL_POS']
	direction = params['DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_thick = params['START_THICK']
	end_thick = params['END_THICK']
	total_thick = params['TOTAL_THICK']
	diameter = params['DIAMETER']
	head_diameter = params['HEAD_DIAMETER']
	head_height = params['HEAD_HEIGHT']
	chamfer_height = params['CHAMFER_HEIGHT']
	chamfer_angle = params['CHAMFER_ANGLE']
	clearance_under_head = params['CLEARANCE_UNDER_HEAD']
	head_diameter_clearance = params['HEAD_DIAMETER_CLEARANCE']
	first_plate_clearance = params['FIRST_PLATE_CLEARANCE']
	clearance_between_plates = params['CLEARANCE_BETWEEN_PLATES']
	second_plate_clearance = params['SECOND_PLATE_CLEARANCE']
	clearance_hole_extra_end_plate = params['CLEARANCE_HOLE_EXTRA_END_PLATE']
	check_hole_extra_end_plate = params['CHECK_HOLE_EXTRA_END_PLATE']
	length = params['LENGTH']
	post_end_plate_thick = params['POST_END_PLATE_THICK']
	groove_distance_tip = params['GROOVE_DISTANCE_TIP']
	groove_distance_between = params['GROOVE_DISTANCE_BETWEEN']
	groove_radius = params['GROOVE_RADIUS']
	groove_width = params['GROOVE_WIDTH']
	groove_amount = params['GROOVE_AMOUNT']
	radius = params['RADIUS']

	

	tmp_pos = VISIPoint.New()
	x_axis = VISIVector.New()
	rev_dir = VISIVector.New()
	Bool_List = []
	UniList1 =[]
	UniList2 =[]
	Result_List = []

	## calculate start point, x axis and reversed direction
	VISIGeoLib.ProjectPointOntoVector(position, direction, clearance_under_head, tmp_pos)
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	rev_dir.Set(- direction.Cx(), - direction.Cy(), - direction.Cz())

	## create the column head
	head = VISISolidFactory.CreateCylinder(head_diameter / 2, head_height, tmp_pos, x_axis, direction)
	Bool_List.append(head)

	VISIGeoLib.ProjectPointOntoVector(tmp_pos, direction, head_height, tmp_pos)

	## create the column body
	body = VISISolidFactory.CreateCylinder(diameter / 2, length - head_height - chamfer_height,
					       tmp_pos, x_axis, direction)
	Bool_List.append(body)

	VISIGeoLib.ProjectPointOntoVector(position, direction, length + clearance_under_head, tmp_pos)

	## calculate the minimum radius of the column cone
	r_cone = (diameter / 2) - chamfer_height * math.tan(chamfer_angle)

	## create the column cone
	cone = VISISolidFactory.CreateCone(r_cone, chamfer_height, chamfer_angle, tmp_pos, x_axis, rev_dir)
	
	## Create chamfer at the tip
	edges = [12]
	VISISolidFactory.BlendEdges(cone, edges, radius, 1)
	
	Bool_List.append(cone)

	## unite all the column parts
	
	#Creat grooves
	if groove_amount:
		VISISolidFactory.Unite(Bool_List, UniList1)
		
		#create first groove
		VISIGeoLib.ProjectPointOntoVector(tmp_pos, direction, -groove_distance_tip, tmp_pos)
		diameter_tor=diameter/2 + math.sqrt(math.pow(groove_radius,2)-math.pow((groove_width/2),2))
		tor1 = VISISolidFactory.CreateTorus(diameter_tor, groove_radius, tmp_pos, x_axis, direction)
		UniList1.append(tor1)
		
		#create the other x grooves
		x=1./1000
		while x < groove_amount:
			VISIGeoLib.ProjectPointOntoVector(tmp_pos, direction, -groove_distance_between, tmp_pos)
			tor = VISISolidFactory.CreateTorus(diameter_tor, groove_radius, tmp_pos, x_axis, direction)
			UniList1.append(tor)
			x=x +1./1000
		
		VISISolidFactory.Subtract(UniList1, Result_List)
	else:
		VISISolidFactory.Unite(Bool_List, Result_List)
	
	return Result_List[0]

def CreateCavity (elem_tag):
	position = params['POSITION']
	sel_pos = params['SEL_POS']
	direction = params['DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_thick = params['START_THICK']
	end_thick = params['END_THICK']
	total_thick = params['TOTAL_THICK']
	diameter = params['DIAMETER']
	head_diameter = params['HEAD_DIAMETER']
	head_height = params['HEAD_HEIGHT']
	chamfer_height = params['CHAMFER_HEIGHT']
	chamfer_angle = params['CHAMFER_ANGLE']
	clearance_under_head = params['CLEARANCE_UNDER_HEAD']
	head_diameter_clearance = params['HEAD_DIAMETER_CLEARANCE']
	first_plate_clearance = params['FIRST_PLATE_CLEARANCE']
	clearance_between_plates = params['CLEARANCE_BETWEEN_PLATES']
	second_plate_clearance = params['SECOND_PLATE_CLEARANCE']
	clearance_hole_extra_end_plate = params['CLEARANCE_HOLE_EXTRA_END_PLATE']
	check_hole_extra_end_plate = params['CHECK_HOLE_EXTRA_END_PLATE']
	length = params['LENGTH']
	post_end_plate_thick = params['POST_END_PLATE_THICK']

	tmp_pos = VISIPoint.New()
	x_axis = VISIVector.New()
	Bool_List = []
	Result_List = []

	## calculate start point and x axis
	VISIGeoLib.ProjectPointOntoVector(position, direction, clearance_under_head, tmp_pos)
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)

	## create the head cavity
	head_cav = VISISolidFactory.CreateCylinder((head_diameter + head_diameter_clearance) / 2,
						   head_height + clearance_under_head, position, x_axis, direction)
	Bool_List.append(head_cav)
	cam_list = [head_diameter + head_diameter_clearance, head_height + clearance_under_head]
	VISI.AttachCamAttributes(head_cav,cam_list,"PCC13_1",elem_tag,params["BUILDER_UNIT"])

	VISIGeoLib.ProjectPointOntoVector(position, direction, clearance_under_head + head_height, tmp_pos)

	## create the cavity in the start plate
	start_plate_cav = VISISolidFactory.CreateCylinder((diameter + first_plate_clearance) / 2,
							  start_thick - (head_height + clearance_under_head),
							  tmp_pos, x_axis, direction)
	Bool_List.append(start_plate_cav)
	cam_list = [diameter + first_plate_clearance, start_thick - (head_height + clearance_under_head)]
	VISI.AttachCamAttributes(start_plate_cav,cam_list,"PCC13_2",elem_tag,params["BUILDER_UNIT"])

	VISIGeoLib.ProjectPointOntoVector(position, direction, start_thick, tmp_pos)

	## check if the cavity between the start and the end plates has to be made
	if (total_thick - (start_thick + end_thick) > 0):
		## create the cavity between start and end plates	
		between_plates_cav = VISISolidFactory.CreateCylinder((diameter + clearance_between_plates) / 2,
								     total_thick - (start_thick + end_thick),
								     tmp_pos, x_axis, direction)
		Bool_List.append(between_plates_cav)
		cam_list = [diameter + clearance_between_plates, total_thick - (start_thick + end_thick)]
		VISI.AttachCamAttributes(between_plates_cav,cam_list,"PCC13_3",elem_tag,params["BUILDER_UNIT"])

		VISIGeoLib.ProjectPointOntoVector(tmp_pos, direction, total_thick - (start_thick + end_thick), tmp_pos)

	## create the cavity in the end plate
	end_plate_cav = VISISolidFactory.CreateCylinder((diameter + second_plate_clearance) / 2, end_thick,
							tmp_pos, x_axis, direction)
	Bool_List.append(end_plate_cav)
	if (total_thick - (start_thick + end_thick) > 0):
		cam_list = [diameter + second_plate_clearance, end_thick]
		VISI.AttachCamAttributes(end_plate_cav,cam_list,"PCC13_4",elem_tag,params["BUILDER_UNIT"])
	else:
		cam_list = [diameter + second_plate_clearance, end_thick]
		VISI.AttachCamAttributes(end_plate_cav,cam_list,"PCC13_3",elem_tag,params["BUILDER_UNIT"])
		
	if (check_hole_extra_end_plate and post_end_plate_thick.__len__()):

		VISIGeoLib.ProjectPointOntoVector(position, direction, total_thick, tmp_pos)
		
		## create the extra end plate cavity
		extra_end_plate_cav = VISISolidFactory.CreateCylinder((diameter + clearance_hole_extra_end_plate) / 2,
								      post_end_plate_thick[0], tmp_pos, x_axis, direction)
		Bool_List.append(extra_end_plate_cav)
		if (total_thick - (start_thick + end_thick) > 0):
			cam_list = [diameter + clearance_hole_extra_end_plate, post_end_plate_thick[0]]
			VISI.AttachCamAttributes(extra_end_plate_cav,cam_list,"PCC13_5",elem_tag,params["BUILDER_UNIT"])
		else:
			cam_list = [diameter + clearance_hole_extra_end_plate, post_end_plate_thick[0]]
			VISI.AttachCamAttributes(extra_end_plate_cav,cam_list,"PCC13_4",elem_tag,params["BUILDER_UNIT"])

	## unite all the cavity parts
	VISISolidFactory.Unite(Bool_List, Result_List,0)
	
	return Result_List[0]

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_pcc13s'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, Xaxis)

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
