import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib, math, VISIParam
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	thr_diameter = params['THR_DIAMETER']
	diameter = params['DIAMETER']
	head_diameter = params['HEAD_DIAMETER']
	head_height = params['HEAD_HEIGHT']
	key_diameter = params['KEY_DIAMETER']
	disc_height = params['DISC_HEIGHT']
	emb_length = params['EMB_LENGTH']
	length = params['LENGTH']
	head_clearance = params['HEAD_CLEARANCE']
	pre_hole_diam = params['PRE_HOLE_DIAM']
	thr_dep_screw = params['THR_DEP_SCREW']
	hole_dep_screw = params['HOLE_DEP_SCREW']
	thread_pitch = params['THREAD_PITCH']
	housing_height = params['HOUSING_HEIGHT']
	create_hexagon = params['CREATE_HEXAGON']

	#direction.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	body_list = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())

	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, housing_height, start_pos)	
	head = VISISolidFactory.CreateCylinder(head_diameter / 2, head_height, start_pos, x_axis, direction)
	body_list.append(head)

	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, head_height + 0.0002, start_pos)
	curing = VISISolidFactory.CreateCylinder(head_diameter / 2, disc_height, start_pos, x_axis, direction)
	body_list.append(curing)
	
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, -0.0002, start_pos)
	body1 = VISISolidFactory.CreateCylinder(diameter / 2, 0.002, start_pos, x_axis, direction)
	body_list.append(body1)
	
	# edges = [12]
	# VISISolidFactory.ChamferEdges(body1, edges, 0.002, 0.002, 1)
	
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, 0.002, start_pos)
	body2 = VISISolidFactory.CreateCylinder(diameter / 2, emb_length - 0.002, start_pos, x_axis, direction)
	body_list.append(body2)

	## attach Thread attribute
	VISI.AttachThreadAttributes(body2, thread_pitch)

	VISISolidFactory.Unite(body_list, Result_List)

	
	edges = [30]
	VISISolidFactory.ChamferEdges(Result_List[0], edges, 0.0005, 0.0005, 1)
	
	if create_hexagon:
		body_list=[]
		start_pos.Set(position.x(), position.y(), position.z())
		VISIGeoLib.ProjectPointOntoVector(start_pos, direction, housing_height, start_pos)
		#Es wird der Umkreis verwendet
		pri1 = VISISolidFactory.CreatePrism((key_diameter/math.cos(math.radians(30)))/2, head_height, 6, start_pos, x_axis, direction)
		body=Result_List[0]
		body_list.append(body)
		body_list.append(pri1)
		Result_List=[]
		
		VISISolidFactory.Subtract(body_list,Result_List)
		
	return Result_List[0]
	

def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	thr_diameter = params['THR_DIAMETER']
	diameter = params['DIAMETER']
	head_diameter = params['HEAD_DIAMETER']
	head_height = params['HEAD_HEIGHT']
	key_diameter = params['KEY_DIAMETER']
	disc_height = params['DISC_HEIGHT']
	emb_length = params['EMB_LENGTH']
	length = params['LENGTH']
	head_clearance = params['HEAD_CLEARANCE']
	pre_hole_diam = params['PRE_HOLE_DIAM']
	thr_dep_screw = params['THR_DEP_SCREW']
	hole_dep_screw = params['HOLE_DEP_SCREW']
	thread_pitch = params['THREAD_PITCH']
	housing_height = params['HOUSING_HEIGHT']
	start_plate = params['START_PLATE']
	
	#direction.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())

	body_list = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())

	pos = VISIPoint.New()
	pos.Set(position.x(), position.y(), position.z())
	
	head_cav = VISISolidFactory.CreateCylinder((head_diameter + head_clearance)/2, housing_height + head_height + 0.002, start_pos, x_axis, direction)
	#body_list.append(head_cav)
	cam_list = [(head_diameter + head_clearance), housing_height + head_height + 0.002]
        VISI.AttachCamAttributes(head_cav,cam_list,"PLUGCU_1",elem_tag,params["BUILDER_UNIT"])

        if (start_plate):
                cavity_face = VISISolidFactory.GetFaceInBodyFromPoint(head_cav, pos)
                hole = VISIParam.ExtendFaceToSolid(start_plate, cavity_face, direction)
                hole= VISIParam.AdaptCavityToPlate(start_plate, hole, 0)
                body_list.append(hole)
        else:
                body_list.append(head_cav)

	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, housing_height + head_height + 0.002, start_pos)
	body_cav = VISISolidFactory.CreateCylinder(pre_hole_diam/2, emb_length + thr_dep_screw + hole_dep_screw, start_pos, x_axis, direction)
	body_list.append(body_cav)
	cam_list = [pre_hole_diam, emb_length + thr_dep_screw+ hole_dep_screw, diameter, emb_length + thr_dep_screw, thread_pitch]
        
	#Differentiate between metric and inch thread
	if thr_diameter[0] == "M":
		VISI.AttachCamAttributes(body_cav, cam_list, "PLUGCU_21", thr_diameter, elem_tag, params["BUILDER_UNIT"])
	else:
		VISI.AttachCamAttributes(body_cav, cam_list, "PLUGCU_22", thr_diameter, elem_tag, params["BUILDER_UNIT"])

        h_cone = pre_hole_diam/2.* math.tan(math.pi/6)
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, emb_length + thr_dep_screw + hole_dep_screw + h_cone, start_pos)
	cone = VISISolidFactory.CreateCone(0, h_cone, math.pi/3,
                                           start_pos, x_axis, inv_dir)
        body_list.append(cone)
	
	VISISolidFactory.Unite(body_list, Result_List,0)

	return Result_List[0]

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_plugcu_2'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, Xaxis)

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
