import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib, VISIParam, math
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	diameter = params['DIAMETER']
	length = params['LENGTH']
	housing_depth = params['HOUSING_DEPTH']
	
	hole_diameter = 0

	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())

	body_list = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(inv_dir, x_axis)
	

	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, housing_depth, start_pos)	
	plug = VISISolidFactory.CreateCylinder(diameter / 2, length, start_pos, x_axis, direction)


	edges = [6,12]
	VISISolidFactory.ChamferEdges(plug, edges, diameter/20, diameter/20, 1)


	return plug

def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	diameter = params['DIAMETER']
	length = params['LENGTH']
	housing_depth = params['HOUSING_DEPTH']
	start_plate = params['START_PLATE']

	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(inv_dir, x_axis)
	

	pos = VISIPoint.New()
	pos.Set(position.x(), position.y(), position.z())
	
	Uni_list = []
	Result_List = []
	
	plug_cav = VISISolidFactory.CreateCylinder(diameter / 2, housing_depth + length, start_pos, x_axis, direction)
	
	if (start_plate):
                cavity_face = VISISolidFactory.GetFaceInBodyFromPoint(plug_cav, pos)
                hole = VISIParam.ExtendFaceToSolid(start_plate, cavity_face, direction)
                hole = VISIParam.AdaptCavityToPlate(start_plate, hole, 0)
                Uni_list.append(hole)
        else:
				cam_list = [diameter,housing_depth + length]
				VISI.AttachCamAttributes(plug_cav, cam_list, "PLUGS_2_1", elem_tag, params["BUILDER_UNIT"])
				Uni_list.append(plug_cav)
				
	#Create Cone
	h_cone = (diameter/2)/(math.tan(math.radians(60)))
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, housing_depth + length+h_cone, start_pos)	
	cone = VISISolidFactory.CreateCone(0, h_cone,math.radians(60), start_pos, x_axis, inv_dir)
	Uni_list.append(cone)
				
	#Unite
	VISISolidFactory.Unite(Uni_list, Result_List)
	return Result_List[0]

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_plugs_2'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, Xaxis)

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)

