import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib, math, VISIParam
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	thr_diameter = params['THR_DIAMETER']
	diameter = params['DIAMETER']
	key_diameter = params['KEY_DIAMETER']
	length = params['LENGTH']
	housing_height = params['HOUSING_HEIGHT']
	pre_hole_diam = params['PRE_HOLE_DIAM']
	thr_dep_screw = params['THR_DEP_SCREW']
	hole_dep_screw = params['HOLE_DEP_SCREW']
	thread_pitch = params['THREAD_PITCH']
        draw_hexagon = params['DRAW_HEXAGON']
	key_height = params['KEY_HEIGHT']
	
	#direction.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	Uni_list = []
	Result_List = []
	boolean_list = []
	tmp_pos = VISIPoint.New()
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())

	if(housing_height):
		VISIGeoLib.ProjectPointOntoVector(start_pos, direction, housing_height, start_pos)

	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, length/2, tmp_pos)
        cone = VISISolidFactory.CreateCone(diameter/2, length/2, 0.0625, tmp_pos, x_axis, inv_dir)
        Uni_list.append(cone)

        VISIGeoLib.ProjectPointOntoVector(tmp_pos, direction, length/2, tmp_pos)
        cone = VISISolidFactory.CreateCone(diameter/2 - (length/2 * math.tan(0.0625)), length/2, 0.0625, tmp_pos, x_axis, inv_dir)
        Uni_list.append(cone)

        VISISolidFactory.Unite(Uni_list, Result_List)

        plug = Result_List[0]

        if (draw_hexagon):
                hex_radius = key_diameter / 2. / math.sin(math.pi / 3.)
                prism = VISISolidFactory.CreatePrism(hex_radius, key_height, 6, start_pos, x_axis, direction)
                boolean_list.append(plug)
                boolean_list.append(prism)

                VISISolidFactory.Subtract(boolean_list, Result_List)


	return Result_List[0]

def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	thr_diameter = params['THR_DIAMETER']
	diameter = params['DIAMETER']
	key_diameter = params['KEY_DIAMETER']
	length = params['LENGTH']
	housing_diameter = params['HOUSING_DIAMETER']
	housing_height = params['HOUSING_HEIGHT']
	pre_hole_diam = params['PRE_HOLE_DIAM']
	thr_dep_screw = params['THR_DEP_SCREW']
	hole_dep_screw = params['HOLE_DEP_SCREW']
	thread_pitch = params['THREAD_PITCH']
	cyl_hole = params['CYL_HOLE']
	start_plate = params['START_PLATE']
	
	#direction.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	Uni_list = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())

	pos = VISIPoint.New()
	pos.Set(position.x(), position.y(), position.z())

	if(housing_height):
                if(housing_diameter > 0):
                    cav1 = VISISolidFactory.CreateCylinder(housing_diameter/2, housing_height, start_pos, x_axis, direction)
                    cam_list = [housing_diameter, housing_height]
                    VISI.AttachCamAttributes(cav1,cam_list,"PLUG_1",elem_tag,params["BUILDER_UNIT"])
                else:
                    cav1 = VISISolidFactory.CreateCylinder(pre_hole_diam/2, housing_height, start_pos, x_axis, direction)
                    cam_list = [pre_hole_diam, housing_height]
                    VISI.AttachCamAttributes(cav1,cam_list,"PLUG_1",elem_tag,params["BUILDER_UNIT"])
		VISIGeoLib.ProjectPointOntoVector(start_pos, direction, housing_height, start_pos)

		if (start_plate):
                        cavity_face = VISISolidFactory.GetFaceInBodyFromPoint(cav1, pos)
                        hole = VISIParam.ExtendFaceToSolid(start_plate, cavity_face, direction)
                        hole= VISIParam.AdaptCavityToPlate(start_plate, hole, 0)
                        Uni_list.append(hole)
                else:
                        Uni_list.append(cav1)


        cav2 = VISISolidFactory.CreateCylinder(pre_hole_diam/2, length + thr_dep_screw + hole_dep_screw, start_pos, x_axis, direction)
        #Uni_list.append(cav2)
        gap_rad = length/2 * math.tan(0.0625)
        max_diam = diameter + 2*gap_rad
        gap_rad = (length + thr_dep_screw) * math.tan(0.0625)
        min_diam = max_diam - 2*gap_rad

        if (cyl_hole):
            cam_list = [pre_hole_diam, length + thr_dep_screw + hole_dep_screw, diameter, length + thr_dep_screw, thread_pitch]
        else:
            cam_list = [pre_hole_diam, length + thr_dep_screw + hole_dep_screw, min_diam, length + thr_dep_screw, thread_pitch, max_diam]
            
        #Differentiate between metric and inch thread
	if thr_diameter[0] == "M":
		VISI.AttachCamAttributes(cav2, cam_list, "PLUG_1_21", thr_diameter, elem_tag, params["BUILDER_UNIT"])
	else:
		VISI.AttachCamAttributes(cav2, cam_list, "PLUG_1_22", thr_diameter, elem_tag, params["BUILDER_UNIT"])

        if (start_plate) and not(housing_height):
            cavity_face = VISISolidFactory.GetFaceInBodyFromPoint(cav2, pos)
            hole = VISIParam.ExtendFaceToSolid(start_plate, cavity_face, direction)
            hole= VISIParam.AdaptCavityToPlate(start_plate, hole, 0)
            Uni_list.append(hole)
        else:
            Uni_list.append(cav2)
                        
        VISIGeoLib.ProjectPointOntoVector(start_pos, direction, length + thr_dep_screw, start_pos)
                
##        if(cyl_hole):      
##                cav2 = VISISolidFactory.CreateCylinder(pre_hole_diam/2, length + thr_dep_screw + hole_dep_screw, start_pos, x_axis, direction)
##                #Uni_list.append(cav2)
##                gap_rad = length/2 * math.tan(0.0625)
##                max_diam = diameter + 2*gap_rad
##                gap_rad = (length + thr_dep_screw) * math.tan(0.0625)
##                min_diam = max_diam - 2*gap_rad
##                cam_list = [pre_hole_diam, length + thr_dep_screw + hole_dep_screw, min_diam, length + thr_dep_screw, thread_pitch, max_diam]
##                VISI.AttachCamAttributes(cav2, cam_list, "PLUG_1_2", thr_diameter, elem_tag, params["BUILDER_UNIT"])
##
##                if (start_plate) and not(housing_height):
##                        cavity_face = VISISolidFactory.GetFaceInBodyFromPoint(cav2, pos)
##                        hole = VISIParam.ExtendFaceToSolid(start_plate, cavity_face, direction)
##                        hole= VISIParam.AdaptCavityToPlate(start_plate, hole, 0)
##                        Uni_list.append(hole)
##                else:
##                        Uni_list.append(cav2)
##                        
##                VISIGeoLib.ProjectPointOntoVector(start_pos, direction, length + thr_dep_screw, start_pos)
##        else:
##                VISIGeoLib.ProjectPointOntoVector(start_pos, direction, length + thr_dep_screw, start_pos)
##                cone = VISISolidFactory.CreateCone(pre_hole_diam/2, length + thr_dep_screw, 0.0625, start_pos, x_axis, inv_dir)
##                #Uni_list.append(cone)
##
##                if (start_plate) and not(housing_height):
##                        VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, length + thr_dep_screw, start_pos)
##                        cavity_face = VISISolidFactory.GetFaceInBodyFromPoint(cone, start_pos)
##                        hole = VISIParam.ExtendFaceToSolid(start_plate, cavity_face, direction)
##                        hole= VISIParam.AdaptCavityToPlate(start_plate, hole, 0)
##                        Uni_list.append(hole)
##                        VISIGeoLib.ProjectPointOntoVector(start_pos, direction, length + thr_dep_screw, start_pos)
##                else:
##                        Uni_list.append(cone)
##
##                cav2 = VISISolidFactory.CreateCylinder(pre_hole_diam/2, hole_dep_screw, start_pos, x_axis, direction)
##                Uni_list.append(cav2)
##                cam_list = [pre_hole_diam, hole_dep_screw, diameter, hole_dep_screw, thread_pitch]
##                VISI.AttachCamAttributes(cav2, cam_list, "PLUG_1_2", thr_diameter, elem_tag, params["BUILDER_UNIT"])

        h_cone = pre_hole_diam/2.* math.tan(math.pi/6)
        VISIGeoLib.ProjectPointOntoVector(start_pos, direction, hole_dep_screw + h_cone, start_pos)
        cone = VISISolidFactory.CreateCone(0, h_cone, math.pi/3, start_pos, x_axis, inv_dir)
        Uni_list.append(cone)

	VISISolidFactory.Unite(Uni_list, Result_List,0)

	return Result_List[0]

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_plug_1'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, Xaxis)

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
