import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib, math
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	diameter_int = params['DIAMETER_INT']
	length_over_collar = params['LENGTH_OVER_COLLAR']
	length_under_collar = params['LENGTH_UNDER_COLLAR']
	diameter_ext = params['DIAMETER_EXT']
	clear_diam_over_length = params['CLEAR_DIAM_OVER_LENGTH']
	clear_length = params['CLEAR_LENGTH']
	diameter_groove = params['DIAMETER_GROOVE']
	length_groove = params['LENGTH_GROOVE']
	total_length = params['TOTAL_LENGTH']
	end_thick = params['END_THICK']
	
	Uni_list = []
	Groove_list = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())

	#Move
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, end_thick-clear_length, start_pos)

	#Create Body
	#upper part
	cyl1 = VISISolidFactory.CreateCylinder(diameter_ext/2, total_length, start_pos, x_axis, direction)
	Uni_list.append(cyl1)
		
	# #chamfer top
	edges = [6,12]
	VISISolidFactory.ChamferEdges(cyl1, edges, diameter_ext/50, diameter_ext/50, 1)
	
	#hole in middle
	cyl5 = VISISolidFactory.CreateCylinder(diameter_int/2, total_length, start_pos, x_axis, inv_dir)
	Uni_list.append(cyl5)
	
	#Create Groove
	#Middle
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, -length_over_collar, start_pos)
	gro1 = VISISolidFactory.CreateCylinder(diameter_ext/2, length_groove, start_pos, x_axis, inv_dir)
	Groove_list.append(gro1)
	
	gro2 = VISISolidFactory.CreateCylinder(diameter_groove/2, length_groove, start_pos, x_axis, inv_dir)
	Groove_list.append(gro2)
	
	#Subtract
	VISISolidFactory.Subtract(Groove_list, Uni_list)
	Groove_list = []
	
	#Bottom
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, length_over_collar-total_length/2+length_under_collar/2, start_pos)
	gro3 = VISISolidFactory.CreateCylinder(diameter_ext/2, length_groove, start_pos, x_axis, inv_dir)
	Groove_list.append(gro3)
	
	gro4 = VISISolidFactory.CreateCylinder(diameter_groove/2, length_groove, start_pos, x_axis, inv_dir)
	Groove_list.append(gro4)
	
	#Subtract
	VISISolidFactory.Subtract(Groove_list, Uni_list)
	Groove_list = []
	
	#Top
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, -length_under_collar, start_pos)
	gro3 = VISISolidFactory.CreateCylinder(diameter_ext/2, length_groove, start_pos, x_axis, direction)
	Groove_list.append(gro3)
	
	gro4 = VISISolidFactory.CreateCylinder(diameter_groove/2, length_groove, start_pos, x_axis, direction)
	Groove_list.append(gro4)
	
	#Subtract
	VISISolidFactory.Subtract(Groove_list, Uni_list)
	Groove_list = []	
	
	#Subtract
	VISISolidFactory.Subtract(Uni_list, Result_List)
	
	return Result_List[0]

def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	diameter_int = params['DIAMETER_INT']
	length_over_collar = params['LENGTH_OVER_COLLAR']
	length_under_collar = params['LENGTH_UNDER_COLLAR']
	diameter_ext = params['DIAMETER_EXT']
	play_plate = params['PLAY_PLATE']
	dist_between = params['DIST_BETWEEN']
	over_length = params['OVER_LENGTH']
	clear_diam_over_length = params['CLEAR_DIAM_OVER_LENGTH']
	length_in_startplate = params['LENGTH_IN_STARTPLATE']
	length_in_groundplate = params['LENGTH_IN_GROUNDPLATE']
	
	Uni_list = []
	Uni_list2 = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	

	#Create Body
	#upper part
	cyl1 = VISISolidFactory.CreateCylinder(diameter_ext/2, length_in_startplate, start_pos, x_axis, direction)
	cam_list = [diameter_ext, length_in_startplate]
	VISI.AttachCamAttributes(cyl1,cam_list,"RBB3_2_1",elem_tag,params["BUILDER_UNIT"])
	
	Uni_list.append(cyl1)
	
	if over_length:
		VISIGeoLib.ProjectPointOntoVector(start_pos, direction, length_in_startplate, start_pos)
		cyl10 = VISISolidFactory.CreateCylinder(diameter_ext/2, dist_between, start_pos, x_axis, direction)
		cam_list = [diameter_ext, dist_between]
		VISI.AttachCamAttributes(cyl10,cam_list,"RBB3_2_4",elem_tag,params["BUILDER_UNIT"])
		Uni_list.append(cyl10)
		VISIGeoLib.ProjectPointOntoVector(start_pos, direction, dist_between, start_pos)
		cyl11 = VISISolidFactory.CreateCylinder(diameter_ext/2+clear_diam_over_length/2, over_length, start_pos, x_axis, direction)
		cam_list = [diameter_ext+clear_diam_over_length, over_length]
		VISI.AttachCamAttributes(cyl10,cam_list,"RBB3_2_5",elem_tag,params["BUILDER_UNIT"])
		Uni_list.append(cyl11)
		
		VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, dist_between, start_pos)
		VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, length_in_startplate, start_pos)
	
	#lower part
	cyl2 = VISISolidFactory.CreateCylinder(diameter_ext/2, length_in_groundplate, start_pos, x_axis, inv_dir)
	cam_list = [diameter_ext, length_in_groundplate]
	VISI.AttachCamAttributes(cyl2,cam_list,"RBB3_2_2",elem_tag,params["BUILDER_UNIT"])
	
	Uni_list.append(cyl2)
	
	#Unite
	VISISolidFactory.Unite(Uni_list, Uni_list2,0)
	return Uni_list2[0]

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_rbb3_2'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	ref_direction = params['REF_DIRECTION']
	Xaxis.Set(ref_direction.Cx(),ref_direction.Cy(),ref_direction.Cz())

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
