import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib, math
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	diameter = params['DIAMETER']
	length = params['LENGTH']
	ext_diameter = params['EXT_DIAMETER']
	radius = params['RADIUS']
	clear_body = params['CLEAR_BODY']
	ins_depth = params['INS_DEPTH']
	cav_with_cone = params['CAV_WITH_CONE']
	reverse = params['REVERSE']
	

	Uni_list = []
	Uni_list2 = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	end_pos = VISIPoint.New()
	end_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	if reverse:
		direction.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
		inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
		
	#Move to correct position
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, ins_depth, start_pos)
	
	#Create Body
	cyl1 = VISISolidFactory.CreateCylinder(ext_diameter/2, length, start_pos, x_axis, direction)
	Uni_list.append(cyl1)
	
	#Create Hole
	cyl2 = VISISolidFactory.CreateCylinder(diameter/2, length, start_pos, x_axis, direction)
	Uni_list.append(cyl2)
	
	#Subtract
	VISISolidFactory.Subtract(Uni_list, Result_List)
	
	#Blend
	if radius > 0:
		edges = [30]
		VISISolidFactory.BlendEdges(Result_List[0], edges, radius, 1)
		
	#Chamfer
	chamfer_depth = ext_diameter/5

	edges = [31]
	VISISolidFactory.ChamferEdges(Result_List[0], edges, chamfer_depth*math.tan(math.radians(10)), chamfer_depth , 1)
	
	return Result_List[0]
	

	## return the tag of created element, 0 if no element
	return 0

def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	diameter = params['DIAMETER']
	length = params['LENGTH']
	ext_diameter = params['EXT_DIAMETER']
	radius = params['RADIUS']
	clear_body = params['CLEAR_BODY']
	ins_depth = params['INS_DEPTH']
	cav_with_cone = params['CAV_WITH_CONE']
	
	Uni_list = []
	Uni_list2 = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	end_pos = VISIPoint.New()
	end_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	
	#Create Body
	cyl1 = VISISolidFactory.CreateCylinder(ext_diameter/2+clear_body/2, length+ins_depth, start_pos, x_axis, direction)
	Uni_list.append(cyl1)
	
	if cav_with_cone:
		h_cone = (ext_diameter/2+clear_body/2)/(math.tan(math.radians(60))) 
		VISIGeoLib.ProjectPointOntoVector(start_pos, direction, h_cone+ins_depth+length, start_pos)
		cone4 = VISISolidFactory.CreateCone(0, h_cone, math.radians(60), start_pos, x_axis, inv_dir)
		Uni_list.append(cone4)
		
	#Unite
	VISISolidFactory.Unite(Uni_list, Result_List)
		

	## return the tag of created cavity, 0 if no cavity
	return Result_List[0]

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_rbb5_f'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	ref_direction = params['REF_DIRECTION']
	Xaxis.Set(ref_direction.Cx(),ref_direction.Cy(),ref_direction.Cz())

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
