import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib, math
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	inner_diameter = params['INNER_DIAMETER']
	length = params['LENGTH']
	outer_diameter = params['OUTER_DIAMETER']
	upper_chamfer = params['UPPER_CHAMFER']
	lower_chamfer = params['LOWER_CHAMFER']
	length_lower_cavity = params['LENGTH_LOWER_CAVITY']
	radius_blend = params['RADIUS_BLEND']
	angle_chamfer = params['ANGLE_CHAMFER']
	clearence_diameter = params['CLEARENCE_DIAMETER']
	inverse_insertion = params['INVERSE_INSERTION']
	diameter_master_elem = params['DIAMETER_MASTER_ELEM']
	insertion_depth = params['INSERTION_DEPTH']
	plate_thick = params['PLATE_THICK']
	
	Uni_list = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	dir = VISIVector.New()
	dir.Set(direction.Cx(), direction.Cy(), direction.Cz())
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	if inverse_insertion:
		dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
		inv_dir.Set(-inv_dir.Cx(), -inv_dir.Cy(), -inv_dir.Cz())
		VISIGeoLib.ProjectPointOntoVector(start_pos, dir, plate_thick, start_pos)
		
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, insertion_depth, start_pos)
	
	
	cyl1 = VISISolidFactory.CreateCylinder(outer_diameter/2, length, start_pos, x_axis, inv_dir)
	Uni_list.append(cyl1)
	
	
	edges1 = [6]
	chamfer_width= math.tan(angle_chamfer)*lower_chamfer
	VISISolidFactory.ChamferEdges(cyl1, edges1, lower_chamfer, chamfer_width, 1)
	
	edges2 = [12]
	VISISolidFactory.ChamferEdges(cyl1, edges2, upper_chamfer, upper_chamfer, 1)
		
	cyl2 = VISISolidFactory.CreateCylinder(inner_diameter/2, length, start_pos, x_axis, inv_dir)
	Uni_list.append(cyl2)
	
	VISISolidFactory.Subtract(Uni_list, Result_List)
	
	edges3 = [78,79]
	VISISolidFactory.BlendEdges(Result_List[0], edges3, radius_blend, 1)

	## return the tag of created element, 0 if no element
	return Result_List[0]

def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	inner_diameter = params['INNER_DIAMETER']
	length = params['LENGTH']
	outer_diameter = params['OUTER_DIAMETER']
	upper_chamfer = params['UPPER_CHAMFER']
	lower_chamfer = params['LOWER_CHAMFER']
	length_lower_cavity = params['LENGTH_LOWER_CAVITY']
	radius_blend = params['RADIUS_BLEND']
	angle_chamfer = params['ANGLE_CHAMFER']
	clearence_diameter = params['CLEARENCE_DIAMETER']
	inverse_insertion = params['INVERSE_INSERTION']
	diameter_master_elem = params['DIAMETER_MASTER_ELEM']
	insertion_depth = params['INSERTION_DEPTH']
	plate_thick = params['PLATE_THICK']
	
	Uni_list = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	dir = VISIVector.New()
	dir.Set(direction.Cx(), direction.Cy(), direction.Cz())
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	if inverse_insertion:
		dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
		inv_dir.Set(-inv_dir.Cx(), -inv_dir.Cy(), -inv_dir.Cz())
		VISIGeoLib.ProjectPointOntoVector(start_pos, dir, plate_thick, start_pos)
		
	# VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, insertion_depth, start_pos)
	
	
	cyl1 = VISISolidFactory.CreateCylinder((outer_diameter + clearence_diameter)/2, length+length_lower_cavity+insertion_depth, start_pos, x_axis, inv_dir)
	cam_list1 = [outer_diameter + clearence_diameter, length+length_lower_cavity+insertion_depth]
	VISI.AttachCamAttributes(cyl1,cam_list1,"SGB2_1",elem_tag,params["BUILDER_UNIT"])
	
	
	return cyl1

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_sgb2'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	ref_direction = params['REF_DIRECTION']
	Xaxis.Set(ref_direction.Cx(),ref_direction.Cy(),ref_direction.Cz())

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
