import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	diameter = params['DIAMETER']
	height = params['HEIGHT']
	diameter_hole = params['DIAMETER_HOLE']
	diameter_sink = params['DIAMETER_SINK']
	depth_sink = params['DEPTH_SINK']
	clear_diam = params['CLEAR_DIAM']
	clear_height = params['CLEAR_HEIGHT']
	grinding_encore = params['GRINDING_ENCORE']
	
	Uni_list = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	#Move to correct position
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, height-clear_height, start_pos)	
	
	#Create Shim as Cylinder
	cyl1 = VISISolidFactory.CreateCylinder(diameter/2, height, start_pos, x_axis, inv_dir)
	Uni_list.append(cyl1)
	
	#Create Hole as Cylinder
	cyl2 = VISISolidFactory.CreateCylinder(diameter_hole/2, height, start_pos, x_axis, inv_dir)
	Uni_list.append(cyl2)
	
	#Create Sinking as Cylinder
	cyl3 = VISISolidFactory.CreateCylinder(diameter_sink/2, depth_sink, start_pos, x_axis, inv_dir)
	Uni_list.append(cyl3)
	
	if grinding_encore:
		cyl4 = VISISolidFactory.CreateCylinder(diameter/2, grinding_encore, start_pos, x_axis, inv_dir)
		Uni_list.append(cyl4)
	
	
	#Subtract
	VISISolidFactory.Subtract(Uni_list, Result_List)
	
	## return the tag of created element, 0 if no element
	return Result_List[0]

def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	diameter = params['DIAMETER']
	height = params['HEIGHT']
	diameter_hole = params['DIAMETER_HOLE']
	diameter_sink = params['DIAMETER_SINK']
	depth_sink = params['DEPTH_SINK']
	clear_diam = params['CLEAR_DIAM']
	clear_height = params['CLEAR_HEIGHT']

	Uni_list = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	#Move to correct position
	#VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, height, start_pos)	
	
	#Create Shim as Cylinder
	cyl1 = VISISolidFactory.CreateCylinder(diameter/2+clear_diam/2, clear_height, start_pos, x_axis, inv_dir)
	Uni_list.append(cyl1)
	
	cam_list = [diameter/2+clear_diam/2, clear_height]
	VISI.AttachCamAttributes(cyl1,cam_list,"SHIM1_1",elem_tag,params["BUILDER_UNIT"])
	
	
	## return the tag of created element, 0 if no element
	return cyl1

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_shim1'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	ref_direction = params['REF_DIRECTION']
	Xaxis.Set(ref_direction.Cx(),ref_direction.Cy(),ref_direction.Cz())

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
