import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	width = params['WIDTH']
	length = params['LENGTH']
	height = params['HEIGHT']
	radius = params['RADIUS']
	dist_holes = params['DIST_HOLES']
	screw_diam = params['SCREW_DIAM']
	diam_hole = params['DIAM_HOLE']
	diam_sink = params['DIAM_SINK']
	depth_sink = params['DEPTH_SINK']
	clear_length = params['CLEAR_LENGTH']
	clear_width = params['CLEAR_WIDTH']
	rad_cav = params['RAD_CAV']
	ins_depth = params['INS_DEPTH']
	rot_angle = params['ROT_ANGLE']
	grinding_encore = params['GRINDING_ENCORE']

	Uni_list = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())

	if(rot_angle <> 0):
		VISIGeoLib.VectorRotate(x_axis, direction, x_axis, rot_angle)
		VISIGeoLib.VectorRotate(y_axis, direction, y_axis, rot_angle)
	
	#Move to correct position
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, height-ins_depth, start_pos)	
	
	#Create Shim as Block
	block1 = VISISolidFactory.CreateBlock(length,width, height, start_pos, x_axis, inv_dir)
	Uni_list.append(block1)
	
	if grinding_encore:
		block2 = VISISolidFactory.CreateBlock(length,width, grinding_encore, start_pos, x_axis, inv_dir)
		Uni_list.append(block2)
	

	#Blend
	edges = [18,20,22,24]
	VISISolidFactory.BlendEdges(block1, edges, radius, 1)
	
	#Create Holes
	#Create Holse as Cylinder
	
	#Move to correct position
	VISIGeoLib.ProjectPointOntoVector(start_pos, x_axis, dist_holes/2, start_pos)	
	
	cyl2 = VISISolidFactory.CreateCylinder(diam_hole/2, height, start_pos, x_axis, inv_dir)
	Uni_list.append(cyl2)
	
	#Create Sinking as Cylinder
	cyl3 = VISISolidFactory.CreateCylinder(diam_sink/2, depth_sink, start_pos, x_axis, inv_dir)
	Uni_list.append(cyl3)
	
	#Move to correct position
	VISIGeoLib.ProjectPointOntoVector(start_pos, x_axis, -dist_holes, start_pos)	
	
	cyl4 = VISISolidFactory.CreateCylinder(diam_hole/2, height, start_pos, x_axis, inv_dir)
	Uni_list.append(cyl4)
	
	#Create Sinking as Cylinder
	cyl5 = VISISolidFactory.CreateCylinder(diam_sink/2, depth_sink, start_pos, x_axis, inv_dir)
	Uni_list.append(cyl5)
	
	#Subtract
	VISISolidFactory.Subtract(Uni_list, Result_List)
	
	## return the tag of created element, 0 if no element
	return Result_List[0]

def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	width = params['WIDTH']
	length = params['LENGTH']
	height = params['HEIGHT']
	radius = params['RADIUS']
	dist_holes = params['DIST_HOLES']
	screw_diam = params['SCREW_DIAM']
	diam_hole = params['DIAM_HOLE']
	diam_sink = params['DIAM_SINK']
	depth_sink = params['DEPTH_SINK']
	clear_length = params['CLEAR_LENGTH']
	clear_width = params['CLEAR_WIDTH']
	rad_cav = params['RAD_CAV']
	ins_depth = params['INS_DEPTH']
	rot_angle = params['ROT_ANGLE']

	Uni_list = []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())

	if(rot_angle <> 0):
		VISIGeoLib.VectorRotate(x_axis, direction, x_axis, rot_angle)
		VISIGeoLib.VectorRotate(y_axis, direction, y_axis, rot_angle)
	
	
	#Create Shim as Block
	block1 = VISISolidFactory.CreateBlock(length+clear_length,width+clear_width, ins_depth, start_pos, x_axis, inv_dir)

	#Blend
	edges = [18,20,22,24]
	VISISolidFactory.BlendEdges(block1, edges, rad_cav, 1)
	
	return block1

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_shim2'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	ref_direction = params['REF_DIRECTION']
	Xaxis.Set(ref_direction.Cx(),ref_direction.Cy(),ref_direction.Cz())

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
