import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib, VISIParam, math
variables = 0
params = 0
key_name = 0

def CreateElement ():
	sel_pos = params['SEL_POS']
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	col_direction = params['COL_DIRECTION']
	stroke_direction = params['STROKE_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_thick = params['START_THICK']
	end_thick = params['END_THICK']
	total_thick = params['TOTAL_THICK']
	diameter = params['DIAMETER']
	chamfer_height = params['CHAMFER_HEIGHT']
	chamfer_angle = params['CHAMFER_ANGLE']
	length = params['LENGTH']
	angle = params['ANGLE']
	wished_stroke = params['WISHED_STROKE']
	stroke = params['STROKE']
	open_delay = params['OPEN_DELAY']
	clearance_body_diameter = params['CLEARANCE_BODY_DIAMETER']
	clearance_head_diameter = params['CLEARANCE_HEAD_DIAMETER']
	clearance_under_head = params['CLEARANCE_UNDER_HEAD']
	clear_body_diam_end = params['CLEAR_BODY_DIAM_END']
	slot_cavity_x = params['SLOT_CAVITY_X']
	slot_cavity_y = params['SLOT_CAVITY_Y']
	slot_cavity_z = params['SLOT_CAVITY_Z']
	slot_cavity_blend = params['SLOT_CAVITY_BLEND']
	slot_cavity_offset = params['SLOT_CAVITY_OFFSET']
	slot_cavity_angle = params['SLOT_CAVITY_ANGLE']
	hex_width = params['HEX_WIDTH']
	hex_depth = params['HEX_DEPTH']
	
	tmp_pos = VISIPoint.New()
	inverted_direction = VISIVector.New()
	Bool_List = []
	Result_List = []

	inverted_direction.Set(- col_direction.Cx(), - col_direction.Cy(), - col_direction.Cz())

	cone_angle = math.radians(18)

	## calculate the start position of the column
	
	VISIGeoLib.ProjectPointOntoVector(position, col_direction, - (total_thick - end_thick - clearance_under_head),
					  tmp_pos)
					  
	## create the column main body
	body = VISISolidFactory.CreateCylinder(diameter / 2, length - chamfer_height,
					       tmp_pos, ref_direction, col_direction)
	Bool_List.append(body)

	## calculate the minor diameter of the chamfer
	chamfer_min_rad = diameter / 2 - chamfer_height * math.tan(chamfer_angle)

	VISIGeoLib.ProjectPointOntoVector(tmp_pos, col_direction, length , tmp_pos)

	chamfer = VISISolidFactory.CreateCone(chamfer_min_rad, chamfer_height, chamfer_angle,
					      tmp_pos, ref_direction, inverted_direction)
	Bool_List.append(chamfer)
	
	## unite all the column parts
	VISISolidFactory.Unite(Bool_List, Result_List)
		
	
	if hex_width:
		Bool_List = []
		Bool_List.append(Result_List[0])
		Result_List = []
		pri1 = VISISolidFactory.CreatePrism(math.tan(math.radians(30))*(hex_width), hex_depth, 6, tmp_pos, ref_direction, inverted_direction)
		Bool_List.append(pri1)
		VISISolidFactory.Subtract(Bool_List, Result_List)
	return Result_List[0]

def CreateCavity (elem_tag):
	sel_pos = params['SEL_POS']
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	col_direction = params['COL_DIRECTION']
	stroke_direction = params['STROKE_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_thick = params['START_THICK']
	end_thick = params['END_THICK']
	total_thick = params['TOTAL_THICK']
	diameter = params['DIAMETER']
	chamfer_height = params['CHAMFER_HEIGHT']
	chamfer_angle = params['CHAMFER_ANGLE']
	length = params['LENGTH']
	angle = params['ANGLE']
	wished_stroke = params['WISHED_STROKE']
	stroke = params['STROKE']
	open_delay = params['OPEN_DELAY']
	clearance_body_diameter = params['CLEARANCE_BODY_DIAMETER']
	clearance_head_diameter = params['CLEARANCE_HEAD_DIAMETER']
	clearance_under_head = params['CLEARANCE_UNDER_HEAD']
	clear_body_diam_end = params['CLEAR_BODY_DIAM_END']
	slot_cavity_x = params['SLOT_CAVITY_X']
	slot_cavity_y = params['SLOT_CAVITY_Y']
	slot_cavity_z = params['SLOT_CAVITY_Z']
	slot_cavity_blend = params['SLOT_CAVITY_BLEND']
	slot_cavity_offset = params['SLOT_CAVITY_OFFSET']
	slot_cavity_angle = params['SLOT_CAVITY_ANGLE']
	blend_on_bottom = params['BLEND_ON_BOTTOM']
	

	tmp_pos = VISIPoint.New()
	adapt_pos = VISIPoint.New()
	slot_cav_pos = VISIPoint.New()
	inverted_direction = VISIVector.New()
	Bool_List = []
	Result_List = []
	incline_face_list = []

	inverted_direction.Set(- col_direction.Cx(), - col_direction.Cy(), - col_direction.Cz())

	VISIGeoLib.ProjectPointOntoVector(position, col_direction, - (total_thick - end_thick - clearance_under_head), tmp_pos)
	
	cav = VISISolidFactory.CreateCylinder((diameter + clearance_body_diameter) / 2,
					      total_thick - end_thick - clearance_under_head,
					      tmp_pos, ref_direction, col_direction)
	cam_list = [diameter + clearance_body_diameter, total_thick - end_thick - clearance_under_head]
	VISI.AttachCamAttributes(cav, cam_list, "SLDP3_1", elem_tag, params["BUILDER_UNIT"])

        VISIGeoLib.ProjectPointOntoVector(tmp_pos, col_direction, total_thick - end_thick - clearance_under_head, tmp_pos)
	cavity_face = VISISolidFactory.GetFaceInBodyFromPoint(cav, tmp_pos)
	cav = VISIParam.ExtendFaceToSolid(end_plate, cavity_face, col_direction)
	cav = VISIParam.AdaptCavityToPlate(end_plate, cav, 1)

        Bool_List.append(cav)
        
        VISIGeoLib.ProjectPointOntoVector(position, col_direction, 0, tmp_pos)
	
	## create the cavity in the end plate
	cav = VISISolidFactory.CreateCylinder((diameter + clear_body_diam_end) / 2, end_thick,
					      tmp_pos, ref_direction, col_direction)

	## keep the adapting position
	adapt_pos.Set(tmp_pos.x(), tmp_pos.y(), tmp_pos.z())

	## make cavity both with open delay and diameter clearance on end plate
	if ((open_delay != 0) and (open_delay > clear_body_diam_end)):

		## create new x axis and boolean list for local operations
		tmp_x_axis = VISIVector.New()
		Local_Bool_List = []

		## keep the just maden cavity cylinder on end plate
		## that will be used to build the cavity with open delay
		Local_Bool_List.append(cav)

		## calculate the local x axis and the position to build the open delay block
		VISIGeoLib.naXb(col_direction, ref_direction, tmp_x_axis)
		VISIGeoLib.ProjectPointOntoVector(tmp_pos, tmp_x_axis, (open_delay - clear_body_diam_end) / 2, tmp_pos)

		## create the block of the open delay cavity
		cav = VISISolidFactory.CreateBlock(open_delay - clear_body_diam_end,
						   diameter + clear_body_diam_end, end_thick,
						   tmp_pos, tmp_x_axis, col_direction)
		Local_Bool_List.append(cav)

		VISIGeoLib.ProjectPointOntoVector(tmp_pos, tmp_x_axis,
						  (open_delay - clear_body_diam_end) / 2, tmp_pos)

		## create the cylinder of the open delay cavity
		cav = VISISolidFactory.CreateCylinder((diameter + clear_body_diam_end) / 2, end_thick,
						      tmp_pos, tmp_x_axis, col_direction)
		Local_Bool_List.append(cav)

		## unite all the parts of the open delay cavity
		VISISolidFactory.Unite(Local_Bool_List, Result_List)
		## keep the resulting solid on the variable "cav"
		cav = Result_List[0]
		## reset the result list
		Result_List = []

	## adapt the end cavity to the end plate
	cavity_face = VISISolidFactory.GetFaceInBodyFromPoint(cav, adapt_pos)
	cav = VISIParam.ExtendFaceToSolid(end_plate, cavity_face, inverted_direction)

	VISIGeoLib.ProjectPointOntoVector(adapt_pos, col_direction, end_thick, adapt_pos)

	## adapt the end cavity to the end plate
	cavity_face = VISISolidFactory.GetFaceInBodyFromPoint(cav, adapt_pos)
	cav = VISIParam.ExtendFaceToSolid(end_plate, cavity_face, col_direction)
	cav = VISIParam.AdaptCavityToPlate(end_plate, cav, 0)

	## if no open delay has been used then attach cam attribute to the end cavity
	if (open_delay == 0):
		cam_list = [diameter + clear_body_diam_end, end_thick]
		VISI.AttachCamAttributes(cav, cam_list, "SLDP3_2", elem_tag, params["BUILDER_UNIT"])

	Bool_List.append(cav)

	if((length + clearance_under_head) > total_thick):
		if (slot_cavity_x <>0) and (slot_cavity_y <>0) and (slot_cavity_z <>0):
			VISIGeoLib.ProjectPointOntoVector(adapt_pos, stroke_direction, slot_cavity_offset, slot_cav_pos)
                        slot_cavity = VISISolidFactory.CreateBlock(slot_cavity_y, slot_cavity_x, slot_cavity_z ,
							      slot_cav_pos, ref_direction, direction)
                        if(slot_cavity_angle <> 0):

                            if(angle >=0):
                                VISIGeoLib.ProjectPointOntoVector(slot_cav_pos, stroke_direction, -slot_cavity_x/2, slot_cav_pos)
                                VISIGeoLib.ProjectPointOntoVector(slot_cav_pos, direction, slot_cavity_z/2, slot_cav_pos)
                                slot_cavity_face = VISISolidFactory.GetFaceInBodyFromPoint(slot_cavity, slot_cav_pos)
                                VISIGeoLib.ProjectPointOntoVector(slot_cav_pos, direction, -slot_cavity_z/2, slot_cav_pos)
                                incline_face_list.append(slot_cavity_face)

                            if(angle <0):
                                VISIGeoLib.ProjectPointOntoVector(slot_cav_pos, stroke_direction, slot_cavity_x/2, slot_cav_pos)
                                VISIGeoLib.ProjectPointOntoVector(slot_cav_pos, direction, slot_cavity_z/2, slot_cav_pos)
                                slot_cavity_face = VISISolidFactory.GetFaceInBodyFromPoint(slot_cavity, slot_cav_pos)
                                VISIGeoLib.ProjectPointOntoVector(slot_cav_pos, direction, -slot_cavity_z/2, slot_cav_pos)
                                incline_face_list.append(slot_cavity_face)

                            VISISolidFactory.InclineFaces(slot_cavity, incline_face_list, slot_cav_pos, direction, slot_cavity_angle)

                        if(slot_cavity_blend > 0):
                            edges = []
                            edges = [18, 20, 22, 24]
                            VISISolidFactory.BlendEdges(slot_cavity, edges, slot_cavity_blend, 1)

                        if(blend_on_bottom > 0):
                            
                            edges = []

                            ## calculate the length of the slot cavity in the bottom part
                            slot_cavity_bottom_x = slot_cavity_x - (slot_cavity_z * math.tan(slot_cavity_angle))
                            
                            if (slot_cavity_blend >= slot_cavity_x / 2.):
                                edges = [77, 83]
                            elif ((slot_cavity_blend >= slot_cavity_y / 2.) and (slot_cavity_blend >= slot_cavity_bottom_x / 2.)):
                                edges = [94, 98, 102, 105]
                            elif ((slot_cavity_blend >= slot_cavity_bottom_x / 2.) and (slot_cavity_blend < slot_cavity_y / 2.)):
                                edges = [91, 97]
                            elif ((slot_cavity_blend >= slot_cavity_y / 2.) and (slot_cavity_blend < slot_cavity_bottom_x / 2.)):
                                edges = [77, 100]
                            elif (slot_cavity_angle == 0):
                                edges = [88, 68, 100, 94]
                            elif (slot_cavity_blend > 0):
                                edges = [77, 97, 103, 109]
                            else:
                                edges = [25, 28, 31, 34]
                            VISISolidFactory.BlendEdges(slot_cavity, edges, blend_on_bottom, 1)

                        Bool_List.append(slot_cavity)

	
	## unite all the cavity parts
	VISISolidFactory.Unite(Bool_List, Result_List,0)
	
	return Result_List[0]

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_sldp3'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	ref_direction = params['REF_DIRECTION']
	Xaxis.Set(ref_direction.Cx(),ref_direction.Cy(),ref_direction.Cz())

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
