import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib, math
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	typ = params['TYP']
	diameter = params['DIAMETER']
	distance_front_slider = params['DISTANCE_FRONT_SLIDER']
	total_length = params['TOTAL_LENGTH']
	gap = params['GAP']
	length_body = params['LENGTH_BODY']
	diameter_lock = params['DIAMETER_LOCK']
	distance_lock = params['DISTANCE_LOCK']
	slider_width = params['SLIDER_WIDTH']
	slider_angle = params['SLIDER_ANGLE']
	thread = params['THREAD']
	thread_core = params['THREAD_CORE']
	thread_pitch = params['THREAD_PITCH']
	thread_length = params['THREAD_LENGTH']
	core_length = params['CORE_LENGTH']
	slider_height = params['SLIDER_HEIGHT']
	length_lock = params['LENGTH_LOCK']
	rot_angle = params['ROT_ANGLE']
	
	Uni_list = []
	Uni_list2= []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	if(rot_angle <> 0):
		VISIGeoLib.VectorRotate(x_axis, direction, x_axis, rot_angle+math.radians(180))
		VISIGeoLib.VectorRotate(y_axis, direction, y_axis, rot_angle+math.radians(180))
	
	#Move
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, gap, start_pos)
	
	#Create Body
	cyl1 = VISISolidFactory.CreateCylinder(diameter/2, length_body, start_pos, x_axis, direction)
	Uni_list.append(cyl1)
	
	#Chamfer
	edges = [6,12]
	VISISolidFactory.ChamferEdges(cyl1, edges, diameter/60, diameter/60, 1)
	
	#Move
	VISIGeoLib.ProjectPointOntoVector(start_pos, y_axis, -diameter/6, start_pos)
	
	#Create SliderNose
	bl1 = VISISolidFactory.CreateBlock(slider_width, diameter/3,slider_height*2, start_pos, x_axis, inv_dir)
	bl2 = VISISolidFactory.CreateBlock(slider_width, diameter/3,slider_height, start_pos, x_axis, direction)
	
	VISIGeoLib.ProjectPointOntoVector(start_pos, y_axis, diameter/6, start_pos)
	
	math.tan(slider_angle)*slider_height
	
	body_list = [bl1,bl2]
	VISISolidFactory.RotateBodies (body_list, start_pos, x_axis, slider_angle)
	
	#Cut bl1
	back_bodies_list = []
	front_bodies_list = []		
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir,slider_height, start_pos)
	VISISolidFactory.CutWithPlane(start_pos, inv_dir, bl1, front_bodies_list, back_bodies_list)
	
	VISISolidFactory.DeleteBodies(front_bodies_list)
	bl1 = back_bodies_list[0]
	Uni_list.append(bl1)
	back_bodies_list = []
	front_bodies_list = []
	
	#Cut bl2
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction,slider_height, start_pos)
	VISISolidFactory.CutWithPlane(start_pos, direction, bl2, front_bodies_list, back_bodies_list)
	
	VISISolidFactory.DeleteBodies(front_bodies_list)
	bl2 = back_bodies_list[0]
	Uni_list.append(bl2)
	back_bodies_list = []
	front_bodies_list = []
	
	body_list = [bl1,bl2]
	VISISolidFactory.TranslateBodiesByVector(body_list, y_axis, -math.tan(slider_angle)*slider_height+distance_front_slider)
	
	#Chamfer
	edges = [9,15]
	VISISolidFactory.BlendEdges(bl1, edges, diameter/30, 1)
	
	#Unite
	VISISolidFactory.Unite(Uni_list, Uni_list2)
	
	#Create Thread
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction,length_body, start_pos)
	cyl2 = VISISolidFactory.CreateCylinder(thread_core/2, core_length, start_pos, x_axis, inv_dir)
	Uni_list2.append(cyl2)
	
	cam_list = [thread_core,core_length, thread, thread_length, thread_pitch]
	VISI.AttachCamAttributes(cyl2, cam_list, "TSU1_0_1", 0, params["BUILDER_UNIT"])
	
	h_cone = (thread_core/2)/(math.tan(math.radians(60)))
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, h_cone+core_length, start_pos)
	cone2 = VISISolidFactory.CreateCone(0, h_cone, math.radians(60), start_pos, x_axis, direction)
	Uni_list2.append(cone2)
	VISIGeoLib.ProjectPointOntoVector(start_pos, direction, h_cone+core_length, start_pos)
	
	#Create drilling for lock
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir,length_body-distance_lock, start_pos)
	VISIGeoLib.ProjectPointOntoVector(start_pos, x_axis,diameter/2-length_lock, start_pos)
	cyl3 = VISISolidFactory.CreateCylinder(diameter_lock/2, length_lock, start_pos, inv_dir,x_axis)
	Uni_list2.append(cyl3)
	
	h_cone = (diameter_lock/2)/(math.tan(math.radians(60)))
	VISIGeoLib.ProjectPointOntoVector(start_pos, x_axis, -h_cone, start_pos)
	cone3 = VISISolidFactory.CreateCone(0, h_cone, math.radians(60), start_pos, direction,x_axis)
	Uni_list2.append(cone3)
		
	#Subtract
	VISISolidFactory.Subtract(Uni_list2, Result_List)
	
	return Result_List[0]
	


def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	typ = params['TYP']
	diameter = params['DIAMETER']
	distance_front_slider = params['DISTANCE_FRONT_SLIDER']
	total_length = params['TOTAL_LENGTH']
	gap = params['GAP']
	length_body = params['LENGTH_BODY']
	diameter_lock = params['DIAMETER_LOCK']
	distance_lock = params['DISTANCE_LOCK']
	slider_width = params['SLIDER_WIDTH']
	slider_angle = params['SLIDER_ANGLE']
	thread = params['THREAD']
	thread_core = params['THREAD_CORE']
	thread_pitch = params['THREAD_PITCH']
	thread_length = params['THREAD_LENGTH']
	core_length = params['CORE_LENGTH']

	Uni_list = []
	Uni_list2= []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	#Move
	#VISIGeoLib.ProjectPointOntoVector(start_pos, direction, gap, start_pos)
	
	#Create Body
	cyl1 = VISISolidFactory.CreateCylinder(diameter/2, length_body+gap, start_pos, x_axis, direction)
	Uni_list.append(cyl1)
	
	return cyl1

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_su1_0'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	ref_direction = params['REF_DIRECTION']
	Xaxis.Set(ref_direction.Cx(),ref_direction.Cy(),ref_direction.Cz())

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
