import VISI, VISIPoint, VISIVector, VISISolidFactory, VISIGeoLib, math
variables = 0
params = 0
key_name = 0

def CreateElement ():
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	typ = params['TYP']
	diameter = params['DIAMETER']
	body_length = params['BODY_LENGTH']
	diameter_lock = params['DIAMETER_LOCK']
	distance_lock = params['DISTANCE_LOCK']
	length_lock = params['LENGTH_LOCK']
	thread = params['THREAD']
	thread_core = params['THREAD_CORE']
	thread_pitch = params['THREAD_PITCH']
	slide_width = params['SLIDE_WIDTH']
	slide_height = params['SLIDE_HEIGHT']
	rot_angle = params['ROT_ANGLE']
	
	Uni_list = []
	Uni_list2= []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
		
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	if(rot_angle <> 0):
		VISIGeoLib.VectorRotate(x_axis, direction, x_axis, rot_angle+math.radians(180))
		VISIGeoLib.VectorRotate(y_axis, direction, y_axis, rot_angle+math.radians(180))
	
	#Create Body
	cyl1 = VISISolidFactory.CreateCylinder(diameter/2, body_length, start_pos, x_axis, inv_dir)
	Uni_list.append(cyl1)
	
	#Chamfer
	edges = [6,12]
	VISISolidFactory.ChamferEdges(cyl1, edges, diameter/60, diameter/60, 1)
	
	#Create Groove
	# Body
	bl1 = VISISolidFactory.CreateBlock(slide_width*1.01,diameter*2, slide_height, start_pos, x_axis, inv_dir)
	Uni_list.append(bl1)
	
	# Heel
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir, slide_height, start_pos)
	bl2 = VISISolidFactory.CreateBlock(slide_width+slide_width/4,diameter*2, slide_height/9.5, start_pos, x_axis, direction)
	Uni_list.append(bl2)
	
	
	#Create drilling for lock
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir,-slide_height+distance_lock, start_pos)
	VISIGeoLib.ProjectPointOntoVector(start_pos, x_axis,diameter/2-length_lock, start_pos)
	cyl3 = VISISolidFactory.CreateCylinder(diameter_lock/2, length_lock, start_pos, inv_dir,x_axis)
	Uni_list.append(cyl3)
	
	h_cone = (diameter_lock/2)/(math.tan(math.radians(60)))
	VISIGeoLib.ProjectPointOntoVector(start_pos, x_axis, -h_cone, start_pos)
	cone3 = VISISolidFactory.CreateCone(0, h_cone, math.radians(60), start_pos, direction,x_axis)
	Uni_list.append(cone3)
	
	VISIGeoLib.ProjectPointOntoVector(start_pos, x_axis,-diameter/2+length_lock+h_cone, start_pos)
	VISIGeoLib.ProjectPointOntoVector(start_pos, inv_dir,-distance_lock+body_length, start_pos)
	
	#Create Thread
	cyl2 = VISISolidFactory.CreateCylinder(thread_core/2, body_length-slide_height, start_pos, x_axis, direction)
	Uni_list.append(cyl2)
	
	cam_list = [thread_core,body_length-slide_height, thread, (body_length-slide_height)/3*2, thread_pitch]
	VISI.AttachCamAttributes(cyl2, cam_list, "TSU1_2_1", 0, params["BUILDER_UNIT"])
	
	#Subtract
	VISISolidFactory.Subtract(Uni_list, Result_List)
	
	return Result_List[0]

def CreateCavity (elem_tag):
	position = params['POSITION']
	direction = params['DIRECTION']
	ref_direction = params['REF_DIRECTION']
	start_plate = params['START_PLATE']
	end_plate = params['END_PLATE']
	start_face = params['START_FACE']
	master_elem = params['MASTER_ELEM']
	typ = params['TYP']
	diameter = params['DIAMETER']
	body_length = params['BODY_LENGTH']
	diameter_lock = params['DIAMETER_LOCK']
	distance_lock = params['DISTANCE_LOCK']
	length_lock = params['LENGTH_LOCK']
	thread = params['THREAD']
	thread_core = params['THREAD_CORE']
	thread_pitch = params['THREAD_PITCH']
	slide_width = params['SLIDE_WIDTH']
	slide_height = params['SLIDE_HEIGHT']
	rot_angle = params['ROT_ANGLE']

	Uni_list = []
	Uni_list2= []
	Result_List = []
	start_pos = VISIPoint.New()
	start_pos.Set(position.x(), position.y(), position.z())
	x_axis = VISIVector.New()
	VISIGeoLib.ArbitraryNormalAxis(direction, x_axis)
	y_axis = VISIVector.New()
	VISIGeoLib.naXb(direction, x_axis, y_axis)
	inv_dir = VISIVector.New()
	inv_dir.Set(-direction.Cx(), -direction.Cy(), -direction.Cz())
	
	if(rot_angle <> 0):
		VISIGeoLib.VectorRotate(x_axis, direction, x_axis, rot_angle+math.radians(180))
		VISIGeoLib.VectorRotate(y_axis, direction, y_axis, rot_angle+math.radians(180))
	
	#Create Body
	cyl1 = VISISolidFactory.CreateCylinder(diameter/2, body_length, start_pos, x_axis, inv_dir)
	Uni_list.append(cyl1)
	
	return cyl1

def Execute (names, dictionaries):
	global variables
	global params
	global key_name
	param_key = ''
	if(key_name == 0):
		param_key = 'PARAM_NG_su1_2'
	else:
		param_key = key_name
	params = variables[param_key]

	position = params['POSITION']
	save_pos = VISIPoint.New()
	save_pos.Set(position.x(),position.y(),position.z())
	direction = params['DIRECTION']
	save_dir = VISIVector.New()
	save_dir.Set(direction.Cx(),direction.Cy(),direction.Cz())
	Xaxis = VISIVector.New()
	ref_direction = params['REF_DIRECTION']
	Xaxis.Set(ref_direction.Cx(),ref_direction.Cy(),ref_direction.Cz())

	elem_tag = 0
	if ( params['BUILD_ELEM'] ):
		elem_tag = CreateElement()
	cavity_tag = 0
	if ( params['BUILD_CAVITY'] ):
		cavity_tag = CreateCavity(elem_tag)

	if(elem_tag or cavity_tag or params['REBUILDING_CAVITY']):
		names.append(param_key)

		elem_info = {}
		elem_info['ELEM_TAG'] = elem_tag
		elem_info['CAVITY_TAG'] = cavity_tag
		elem_info['POSITION'] = save_pos
		elem_info['DIRECTION'] = save_dir
		elem_info['REF_DIRECTION'] = Xaxis

		dictionaries.append(elem_info)
